# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from paddlex import create_pipeline

pipeline = create_pipeline(pipeline="OCR")

output = pipeline.predict(
    "./test_samples/general_ocr_002.png",
    use_doc_orientation_classify=False,
    use_doc_unwarping=False,
    use_textline_orientation=False,
)

# output = pipeline.predict(
#     "./test_samples/general_ocr_002.png",
#     use_doc_orientation_classify=False,
#     use_doc_unwarping=False,
#     use_textline_orientation=False,
#     text_rec_score_thresh = 0.5
# )

# output = pipeline.predict(
#     "./test_samples/general_ocr_002.png",
#     use_doc_orientation_classify=False,
#     use_doc_unwarping=False,
#     use_textline_orientation=False,
#     text_det_unclip_ratio=3.0,
#     text_det_limit_side_len=1920
# )

# output = pipeline.predict(
#     "./test_samples/general_ocr_002.png",
#     use_doc_orientation_classify=True,
#     use_doc_unwarping=True,
#     use_textline_orientation=False
# )

# output = pipeline.predict(
#     "./test_samples/general_ocr_003.jpg",
#     use_doc_orientation_classify=False,
#     use_doc_unwarping=False,
#     use_textline_orientation=False
# )

# output = pipeline.predict(
#     "./test_samples/general_ocr_003.jpg",
#     use_doc_orientation_classify=False,
#     use_doc_unwarping=False,
#     use_textline_orientation=True
# )

# output = pipeline.predict(
#     "./test_samples/general_ocr_002_rotate_90.png",
#     use_doc_orientation_classify=False,
#     use_doc_unwarping=False,
#     use_textline_orientation=False
# )

# output = pipeline.predict(
#     "./test_samples/general_ocr_002_rotate_90.png",
#     use_doc_orientation_classify=False,
#     use_doc_unwarping=False,
#     use_textline_orientation=True
# )

# output = pipeline.predict(
#     "./test_samples/general_ocr_002.png")

# output = pipeline.predict("./test_samples/财报1.pdf")

for res in output:
    print(res)
    res.print()
    res.save_to_img("./output")
    res.save_to_json("./output")



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from shutil import copyfile
from typing import List, Optional, Tuple

from paddlex.inference.models.common.tokenizer.tokenizer_utils import (
    PretrainedTokenizer,
)


class LlamaTokenizer(PretrainedTokenizer):
    model_input_names = ["input_ids", "attention_mask", "position_ids"]
    resource_files_names = {
        "vocab_file": "sentencepiece.bpe.model",
    }
    pretrained_resource_files_map = {
        "vocab_file": {
            "__internal_testing__/micro-random-llama": "https://bj.bcebos.com/paddlenlp/models/transformers/llama/sentencepiece.bpe.model",
            "__internal_testing__/tiny-random-llama": "https://bj.bcebos.com/paddlenlp/models/transformers/llama/sentencepiece.bpe.model",
            "facebook/llama-7b": "https://bj.bcebos.com/paddlenlp/models/transformers/llama/sentencepiece.bpe.model",
            "facebook/llama-13b": "https://bj.bcebos.com/paddlenlp/models/transformers/llama/sentencepiece.bpe.model",
            "facebook/llama-30b": "https://bj.bcebos.com/paddlenlp/models/transformers/llama/sentencepiece.bpe.model",
            "facebook/llama-65b": "https://bj.bcebos.com/paddlenlp/models/transformers/llama/sentencepiece.bpe.model",
        },
    }

    pretrained_init_configuration = {
        "__internal_testing__/micro-random-llama": {},
        "__internal_testing__/tiny-random-llama": {},
        "facebook/llama-7b": {},
        "facebook/llama-13b": {},
        "facebook/llama-30b": {},
        "facebook/llama-65b": {},
    }
    padding_side = "left"

    def __init__(
        self,
        vocab_file,
        unk_token="<unk>",
        bos_token="<s>",
        eos_token="</s>",
        add_bos_token=True,
        add_eos_token=False,
        sp_model_kwargs=None,
        decode_with_prefix_space=False,
        **kwargs,
    ):
        self.sp_model_kwargs = {} if sp_model_kwargs is None else sp_model_kwargs
        super().__init__(
            bos_token=bos_token, eos_token=eos_token, unk_token=unk_token, **kwargs
        )

        self.vocab_file = vocab_file
        self.add_bos_token = add_bos_token
        self.add_eos_token = add_eos_token
        self.decode_with_prefix_space = decode_with_prefix_space
        self.sp_model = self.get_spm_processor(kwargs.pop("from_slow", True))

    @property
    def vocab_size(self):
        """Returns vocab size"""
        return self.sp_model.get_piece_size()

    def __len__(self):
        """
        Returns the vocabulary size. added_tokens_encoder has to be added in the sp_model
        """
        added_size = 0

        for id in self.added_tokens_decoder:
            if id >= self.sp_model.get_piece_size():
                added_size += 1

        return self.vocab_size + added_size

    @property
    def bos_token_id(self) -> Optional[int]:
        return self.sp_model.bos_id()

    @property
    def eos_token_id(self) -> Optional[int]:
        return self.sp_model.eos_id()

    def get_spm_processor(self, from_slow=True):
        import sentencepiece as spm
        from sentencepiece import sentencepiece_model_pb2 as model_pb2

        tokenizer = spm.SentencePieceProcessor(**self.sp_model_kwargs)
        if from_slow:  # no dependency on protobuf
            tokenizer.Load(self.vocab_file)
            return tokenizer

        with open(self.vocab_file, "rb") as f:
            sp_model = f.read()
            model = model_pb2.ModelProto.FromString(sp_model)
            normalizer_spec = model_pb2.NormalizerSpec()
            normalizer_spec.add_dummy_prefix = False
            model.normalizer_spec.MergeFrom(normalizer_spec)
            sp_model = model.SerializeToString()
            tokenizer.LoadFromSerializedProto(sp_model)
        return tokenizer

    def get_vocab(self):
        """Returns vocab as a dict"""
        vocab = {self.convert_ids_to_tokens(i): i for i in range(self.vocab_size)}
        vocab.update(self.added_tokens_encoder)
        return vocab

    def _tokenize(self, text):
        """Returns a tokenized string."""
        return self.sp_model.encode(text, out_type=str)

    def _convert_token_to_id(self, token):
        """Converts a token (str) in an id using the vocab."""
        return self.sp_model.piece_to_id(token)

    def _convert_id_to_token(self, index):
        """Converts an index (integer) in a token (str) using the vocab."""
        token = self.sp_model.id_to_piece(index)
        return token

    def convert_tokens_to_string(self, tokens):
        """Converts a sequence of tokens (string) in a single string."""
        current_sub_tokens = []
        out_string = ""
        prev_is_special = False
        for i, token in enumerate(tokens):
            # make sure that special tokens are not decoded using sentencepiece model
            if token in self.all_special_tokens:
                if not prev_is_special and i != 0:
                    out_string += " "
                out_string += self.sp_model.decode(current_sub_tokens) + token
                prev_is_special = True
                current_sub_tokens = []
            else:
                current_sub_tokens.append(token)
                prev_is_special = False
        out_string += self.sp_model.decode(current_sub_tokens)
        return out_string

    def save_vocabulary(
        self, save_directory, filename_prefix: Optional[str] = None
    ) -> Tuple[str]:
        """
        Save the vocabulary and special tokens file to a directory.
        Args:
            save_directory (`str`):
                The directory in which to save the vocabulary.
        Returns:
            `Tuple(str)`: Paths to the files saved.
        """
        if not os.path.isdir(save_directory):
            raise ValueError(
                f"Vocabulary path ({save_directory}) should be a directory"
            )

        out_vocab_file = os.path.join(
            save_directory,
            (filename_prefix + "-" if filename_prefix else "")
            + self.resource_files_names["vocab_file"],
        )

        if os.path.abspath(self.vocab_file) != os.path.abspath(
            out_vocab_file
        ) and os.path.isfile(self.vocab_file):
            copyfile(self.vocab_file, out_vocab_file)
        elif not os.path.isfile(self.vocab_file):
            with open(out_vocab_file, "wb") as fi:
                content_spiece_model = self.sp_model.serialized_model_proto()
                fi.write(content_spiece_model)

        return (out_vocab_file,)

    def build_inputs_with_special_tokens(self, token_ids_0, token_ids_1=None):
        if self.add_bos_token:
            bos_token_ids = [self.bos_token_id]
        else:
            bos_token_ids = []

        output = bos_token_ids + token_ids_0

        if token_ids_1 is not None:
            output = output + token_ids_1

        if self.add_eos_token:
            output = output + [self.eos_token_id]

        return output

    def get_special_tokens_mask(
        self,
        token_ids_0: List[int],
        token_ids_1: Optional[List[int]] = None,
        already_has_special_tokens: bool = False,
    ) -> List[int]:
        """
        Retrieve sequence ids from a token list that has no special tokens added. This method is called when adding
        special tokens using the tokenizer `prepare_for_model` method.
        Args:
            token_ids_0 (`List[int]`):
                List of IDs.
            token_ids_1 (`List[int]`, *optional*):
                Optional second list of IDs for sequence pairs.
            already_has_special_tokens (`bool`, *optional*, defaults to `False`):
                Whether or not the token list is already formatted with special tokens for the model.
        Returns:
            `List[int]`: A list of integers in the range [0, 1]: 1 for a special token, 0 for a sequence token.
        """
        if already_has_special_tokens:
            return super().get_special_tokens_mask(
                token_ids_0=token_ids_0,
                token_ids_1=token_ids_1,
                already_has_special_tokens=True,
            )

        if token_ids_1 is None:
            return [1] + ([0] * len(token_ids_0)) + [1]
        return [1] + ([0] * len(token_ids_0)) + [1, 1] + ([0] * len(token_ids_1)) + [1]

    def create_token_type_ids_from_sequences(
        self, token_ids_0: List[int], token_ids_1: Optional[List[int]] = None
    ) -> List[int]:
        """
        Create a mask from the two sequences passed to be used in a sequence-pair classification task. T5 does not make
        use of token type ids, therefore a list of zeros is returned.

        Args:
            token_ids_0 (`List[int]`):
                List of IDs.
            token_ids_1 (`List[int]`, *optional*):
                Optional second list of IDs for sequence pairs.
        Returns:
            `List[int]`: List of zeros.
        """
        eos = [self.eos_token_id]

        if token_ids_1 is None:
            return len(token_ids_0 + eos) * [0]
        return len(token_ids_0 + eos + token_ids_1 + eos) * [0]



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

import numpy as np
from tritonclient import grpc as triton_grpc

from . import constants


def _create_triton_input(data):
    data = json.dumps(data, separators=(",", ":"))
    data = data.encode("utf-8")
    data = [[data]]
    data = np.array(data, dtype=np.object_)
    return data


def _parse_triton_output(data):
    data = data[0, 0]
    data = data.decode("utf-8")
    data = json.loads(data)
    return data


def triton_request(client, model_name, data, *, request_kwargs=None):
    if request_kwargs is None:
        request_kwargs = {}
    input_ = triton_grpc.InferInput(constants.INPUT_NAME, [1, 1], "BYTES")
    input_.set_data_from_numpy(_create_triton_input(data))
    results = client.infer(model_name, inputs=[input_], **request_kwargs)
    output = results.as_numpy(constants.OUTPUT_NAME)
    return _parse_triton_output(output)



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Distributed utils
"""

# TODO: Support XPU

import paddle

from .common_dist_utils import (
    AllGatherVarlenOp,
    ColumnParallelLinear,
    ColumnSequenceParallelLinear,
    GatherOp,
    RowParallelLinear,
    RowSequenceParallelLinear,
    RRColumnSequenceParallelLinear,
    RRRowSequenceParallelLinear,
    ScatterOp,
    get_hcg,
    mark_as_sequence_parallel_parameter,
    sequence_parallel_sparse_mask_labels,
)

__all__ = [
    "ColumnParallelLinear",
    "ColumnSequenceParallelLinear",
    "RowParallelLinear",
    "RowSequenceParallelLinear",
    "GatherOp",
    "ScatterOp",
    "mark_as_sequence_parallel_parameter",
    "ParallelCrossEntropy",
    "get_rng_state_tracker",
    "parallel_matmul",
    "RRColumnSequenceParallelLinear",
    "RRRowSequenceParallelLinear",
    "AllGatherVarlenOp",
    "sequence_parallel_sparse_mask_labels",
    "get_hcg",
]


def parallel_matmul(
    x,
    y,
    bias=None,
    transpose_y=False,
    tensor_parallel_degree=1,
    tensor_parallel_output=True,
    fuse_linear=False,
    training=None,
):
    """
    Parallel matmul wrapper.

    Args:
        x (Tensor): Input tensor.
        y (Tensor): Weight tensor.
        bias (Tensor, optional): Bias tensor. Default is None.
        transpose_y (bool, optional): Whether to transpose y. Default is False.
        tensor_parallel_degree (int, optional): Tensor parallel degree. Default is 1.
        tensor_parallel_output (bool, optional): Whether to output tensor parallel. Default is True.
        fuse_linear (bool, optional): Whether to fuse linear. Default is False.
        training (bool, optional): Training state. Default is None.
    Returns:
        Tensor: Output tensor.
    """
    if paddle.is_compiled_with_xpu():
        from .common_dist_utils import _parallel_matmul as default_parallel_matmul

        return default_parallel_matmul(
            x,
            y,
            bias=bias,
            transpose_y=transpose_y,
            tensor_parallel_degree=tensor_parallel_degree,
            tensor_parallel_output=tensor_parallel_output,
            fuse_linear=fuse_linear,
        )
    else:
        from .common_dist_utils import _parallel_matmul

    return _parallel_matmul(
        x,
        y,
        bias=bias,
        transpose_y=transpose_y,
        tensor_parallel_degree=tensor_parallel_degree,
        tensor_parallel_output=tensor_parallel_output,
        fuse_linear=fuse_linear,
    )



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from functools import partial

import numpy as np
from PIL import Image, ImageDraw, ImageFont

from ....utils.fonts import PINGFANG_FONT
from ...common.result import (
    BaseCVResult,
    HtmlMixin,
    JsonMixin,
    MarkdownMixin,
    XlsxMixin,
)
from ..layout_parsing.result_v2 import (
    format_centered_by_html,
    format_first_line_func,
    format_image_plain_func,
    format_image_scaled_by_html_func,
    format_text_plain_func,
    format_title_func,
    simplify_table_func,
)

VISUALIZE_INDEX_LABELS = [
    "text",
    "formula",
    "inline_formula",
    "display_formula",
    "algorithm",
    "reference",
    "reference_content",
    "content",
    "abstract",
    "paragraph_title",
    "doc_title",
    "vertical_text",
    "ocr",
]


class PaddleOCRVLBlock(object):
    """PaddleOCRVL Block Class"""

    def __init__(self, label, bbox, content="") -> None:
        """
        Initialize a PaddleOCRVLBlock object.

        Args:
            label (str): Label assigned to the block.
            bbox (list): Bounding box coordinates of the block.
            content (str, optional): Content of the block. Defaults to an empty string.
        """
        self.label = label
        self.bbox = list(map(int, bbox))
        self.content = content
        self.image = None

    def __str__(self) -> str:
        """
        Return a string representation of the block.
        """
        _str = f"\n\n#################\nlabel:\t{self.label}\nbbox:\t{self.bbox}\ncontent:\t{self.content}\n#################"
        return _str

    def __repr__(self) -> str:
        """
        Return a string representation of the block.
        """
        _str = f"\n\n#################\nlabel:\t{self.label}\nbbox:\t{self.bbox}\ncontent:\t{self.content}\n#################"
        return _str


def merge_formula_and_number(formula, formula_number):
    """
    Merge a formula and its formula number for display.

    Args:
        formula (str): The formula string.
        formula_number (str): The formula number string.

    Returns:
        str: The merged formula with tag.
    """
    formula = formula.replace("$$", "")
    merge_formula = r"{} \tag*{{{}}}".format(formula, formula_number)
    return f"$${merge_formula}$$"


def format_chart2table_func(block):
    lines_list = block.content.split("\n")
    # 提取表头和内容
    header = lines_list[0].split("|")
    rows = [line.split("|") for line in lines_list[1:]]
    # 构造HTML表格
    html = "<table border=1 style='margin: auto; width: max-content;'>\n"
    html += (
        "  <thead><tr>"
        + "".join(
            f"<th style='text-align: center;'>{cell.strip()}</th>" for cell in header
        )
        + "</tr></thead>\n"
    )
    html += "  <tbody>\n"
    for row in rows:
        html += (
            "    <tr>"
            + "".join(
                f"<td style='text-align: center;'>{cell.strip()}</td>" for cell in row
            )
            + "</tr>\n"
        )
    html += "  </tbody>\n"
    html += "</table>"
    return html


def format_table_center_func(block):
    tabel_content = block.content
    tabel_content = tabel_content.replace(
        "<table>", "<table border=1 style='margin: auto; width: max-content;'>"
    )
    tabel_content = tabel_content.replace("<th>", "<th style='text-align: center;'>")
    tabel_content = tabel_content.replace("<td>", "<td style='text-align: center;'>")
    return tabel_content


def build_handle_funcs_dict(
    *,
    text_func,
    image_func,
    chart_func,
    table_func,
    formula_func,
    seal_func,
):
    """
    Build a dictionary mapping block labels to their formatting functions.

    Args:
        text_func: Function to format text blocks.
        image_func: Function to format image blocks.
        chart_func: Function to format chart blocks.
        table_func: Function to format table blocks.
        formula_func: Function to format formula blocks.
        seal_func: Function to format seal blocks.

    Returns:
        dict: A mapping from block label to handler function.
    """
    return {
        "paragraph_title": format_title_func,
        "abstract_title": format_title_func,
        "reference_title": format_title_func,
        "content_title": format_title_func,
        "doc_title": lambda block: f"# {block.content}".replace("-\n", "").replace(
            "\n", " "
        ),
        "table_title": text_func,
        "figure_title": text_func,
        "chart_title": text_func,
        "vision_footnote": lambda block: block.content.replace("\n\n", "\n").replace(
            "\n", "\n\n"
        ),
        "text": lambda block: block.content.replace("\n\n", "\n").replace("\n", "\n\n"),
        "ocr": lambda block: block.content.replace("\n\n", "\n").replace("\n", "\n\n"),
        "vertical_text": lambda block: block.content.replace("\n\n", "\n").replace(
            "\n", "\n\n"
        ),
        "reference_content": lambda block: block.content.replace("\n\n", "\n").replace(
            "\n", "\n\n"
        ),
        "abstract": partial(
            format_first_line_func,
            templates=["摘要", "abstract"],
            format_func=lambda l: f"## {l}\n",
            spliter=" ",
        ),
        "content": lambda block: block.content.replace("-\n", "  \n").replace(
            "\n", "  \n"
        ),
        "image": image_func,
        "chart": chart_func,
        "formula": formula_func,
        "display_formula": formula_func,
        "inline_formula": formula_func,
        "table": table_func,
        "reference": partial(
            format_first_line_func,
            templates=["参考文献", "references"],
            format_func=lambda l: f"## {l}",
            spliter="\n",
        ),
        "algorithm": lambda block: block.content.strip("\n"),
        "seal": seal_func,
    }


class PaddleOCRVLResult(BaseCVResult, HtmlMixin, XlsxMixin, MarkdownMixin):
    """
    PaddleOCRVLResult class for holding and formatting OCR/VL parsing results.
    """

    def __init__(self, data) -> None:
        """
        Initializes a new instance of the class with the specified data.

        Args:
            data: The input data for the parsing result.
        """
        super().__init__(data)
        HtmlMixin.__init__(self)
        XlsxMixin.__init__(self)
        MarkdownMixin.__init__(self)
        JsonMixin.__init__(self)

    def _to_img(self) -> dict[str, np.ndarray]:
        """
        Convert the parsing result to a dictionary of images.

        Returns:
            dict: Keys are names, values are numpy arrays (images).
        """
        from ..layout_parsing.utils import get_show_color

        res_img_dict = {}
        model_settings = self["model_settings"]
        if model_settings["use_doc_preprocessor"]:
            for key, value in self["doc_preprocessor_res"].img.items():
                res_img_dict[key] = value
        if self["model_settings"]["use_layout_detection"]:
            res_img_dict["layout_det_res"] = self["layout_det_res"].img["res"]

        # for layout ordering image
        image = Image.fromarray(self["doc_preprocessor_res"]["output_img"][:, :, ::-1])
        draw = ImageDraw.Draw(image, "RGBA")
        font_size = int(0.018 * int(image.width)) + 2
        font = ImageFont.truetype(PINGFANG_FONT.path, font_size, encoding="utf-8")
        parsing_result = self["parsing_res_list"]
        order_index = 0
        for block in parsing_result:
            bbox = block.bbox
            label = block.label
            fill_color = get_show_color(label, False)
            draw.rectangle(bbox, fill=fill_color)
            if label in VISUALIZE_INDEX_LABELS:
                text_position = (bbox[2] + 2, bbox[1] - font_size // 2)
                if int(image.width) - bbox[2] < font_size:
                    text_position = (
                        int(bbox[2] - font_size * 1.1),
                        bbox[1] - font_size // 2,
                    )
                draw.text(text_position, str(order_index + 1), font=font, fill="red")
                order_index += 1

        res_img_dict["layout_order_res"] = image

        return res_img_dict

    def _to_html(self) -> dict[str, str]:
        """
        Converts the prediction to its corresponding HTML representation.

        Returns:
            dict: The str type HTML representation result.
        """
        res_html_dict = {}
        if len(self["table_res_list"]) > 0:
            for sno in range(len(self["table_res_list"])):
                table_res = self["table_res_list"][sno]
                table_region_id = table_res["table_region_id"]
                key = f"table_{table_region_id}"
                res_html_dict[key] = table_res.html["pred"]
        return res_html_dict

    def _to_xlsx(self) -> dict[str, str]:
        """
        Converts the prediction HTML to an XLSX file path.

        Returns:
            dict: The str type XLSX representation result.
        """
        res_xlsx_dict = {}
        if len(self["table_res_list"]) > 0:
            for sno in range(len(self["table_res_list"])):
                table_res = self["table_res_list"][sno]
                table_region_id = table_res["table_region_id"]
                key = f"table_{table_region_id}"
                res_xlsx_dict[key] = table_res.xlsx["pred"]
        return res_xlsx_dict

    def _to_str(self, *args, **kwargs) -> dict[str, str]:
        """
        Converts the instance's attributes to a dictionary and then to a string.

        Args:
            *args: Additional positional arguments passed to the base class method.
            **kwargs: Additional keyword arguments passed to the base class method.

        Returns:
            dict: A dictionary with the instance's attributes converted to strings.
        """
        data = {}
        data["input_path"] = self["input_path"]
        data["page_index"] = self["page_index"]
        model_settings = self["model_settings"]
        data["model_settings"] = model_settings
        if self["model_settings"]["use_doc_preprocessor"]:
            data["doc_preprocessor_res"] = self["doc_preprocessor_res"].str["res"]
        if self["model_settings"]["use_layout_detection"]:
            data["layout_det_res"] = self["layout_det_res"].str["res"]
        return JsonMixin._to_str(data, *args, **kwargs)

    def _to_json(self, *args, **kwargs) -> dict[str, str]:
        """
        Converts the object's data to a JSON dictionary.

        Args:
            *args: Positional arguments passed to the JsonMixin._to_json method.
            **kwargs: Keyword arguments passed to the JsonMixin._to_json method.

        Returns:
            dict: A dictionary containing the object's data in JSON format.
        """
        data = {}
        data["input_path"] = self["input_path"]
        data["page_index"] = self["page_index"]
        model_settings = self["model_settings"]
        data["model_settings"] = model_settings
        if self["model_settings"].get("format_block_content", False):
            original_image_width = self["doc_preprocessor_res"]["output_img"].shape[1]
            format_text_func = lambda block: format_centered_by_html(
                format_text_plain_func(block)
            )
            format_image_func = lambda block: format_centered_by_html(
                format_image_scaled_by_html_func(
                    block,
                    original_image_width=original_image_width,
                )
            )

            if self["model_settings"].get("use_chart_recognition", False):
                format_chart_func = format_chart2table_func
            else:
                format_chart_func = format_image_func

            format_seal_func = format_image_func

            format_table_func = lambda block: "\n" + format_table_center_func(block)
            format_formula_func = lambda block: block.content

            handle_funcs_dict = build_handle_funcs_dict(
                text_func=format_text_func,
                image_func=format_image_func,
                chart_func=format_chart_func,
                table_func=format_table_func,
                formula_func=format_formula_func,
                seal_func=format_seal_func,
            )

        parsing_res_list = self["parsing_res_list"]
        parsing_res_list_json = []
        order_index = 1
        for idx, parsing_res in enumerate(parsing_res_list):
            label = parsing_res.label
            if label in VISUALIZE_INDEX_LABELS:
                order = order_index
                order_index += 1
            else:
                order = None
            res_dict = {
                "block_label": parsing_res.label,
                "block_content": parsing_res.content,
                "block_bbox": parsing_res.bbox,
                "block_id": idx,
                "block_order": order,
            }
            if self["model_settings"].get("format_block_content", False):
                if handle_funcs_dict.get(parsing_res.label):
                    res_dict["block_content"] = handle_funcs_dict[parsing_res.label](
                        parsing_res
                    )
                else:
                    res_dict["block_content"] = parsing_res.content

            parsing_res_list_json.append(res_dict)
        data["parsing_res_list"] = parsing_res_list_json
        if self["model_settings"]["use_doc_preprocessor"]:
            data["doc_preprocessor_res"] = self["doc_preprocessor_res"].json["res"]
        if self["model_settings"]["use_layout_detection"]:
            data["layout_det_res"] = self["layout_det_res"].json["res"]
        return JsonMixin._to_json(data, *args, **kwargs)

    def _to_markdown(self, pretty=True, show_formula_number=False) -> dict:
        """
        Save the parsing result to a Markdown file.

        Args:
            pretty (Optional[bool]): whether to pretty markdown by HTML, default by True.
            show_formula_number (bool): whether to show formula numbers.

        Returns:
            dict: Markdown information with text and images.
        """
        original_image_width = self["doc_preprocessor_res"]["output_img"].shape[1]

        if pretty:
            format_text_func = lambda block: format_centered_by_html(
                format_text_plain_func(block)
            )
            format_image_func = lambda block: format_centered_by_html(
                format_image_scaled_by_html_func(
                    block,
                    original_image_width=original_image_width,
                )
            )
        else:
            format_text_func = lambda block: block.content
            format_image_func = format_image_plain_func

        format_chart_func = (
            format_chart2table_func
            if self["model_settings"]["use_chart_recognition"]
            else format_image_func
        )

        if pretty:
            format_table_func = lambda block: "\n" + format_table_center_func(block)
        else:
            format_table_func = lambda block: simplify_table_func("\n" + block.content)

        format_formula_func = lambda block: block.content
        format_seal_func = format_image_func

        handle_funcs_dict = build_handle_funcs_dict(
            text_func=format_text_func,
            image_func=format_image_func,
            chart_func=format_chart_func,
            table_func=format_table_func,
            formula_func=format_formula_func,
            seal_func=format_seal_func,
        )

        markdown_content = ""
        markdown_info = {}
        markdown_info["markdown_images"] = {}
        for idx, block in enumerate(self["parsing_res_list"]):
            label = block.label
            if block.image is not None:
                markdown_info["markdown_images"][block.image["path"]] = block.image[
                    "img"
                ]
            handle_func = handle_funcs_dict.get(label, None)
            if (
                show_formula_number
                and (label == "display_formula" or label == "formula")
                and idx != len(self["parsing_res_list"]) - 1
            ):
                next_block = self["parsing_res_list"][idx + 1]
                next_block_label = next_block.label
                if next_block_label == "formula_number":
                    block.content = merge_formula_and_number(
                        block.content, next_block.content
                    )
            if handle_func:
                markdown_content += (
                    "\n\n" + handle_func(block)
                    if markdown_content
                    else handle_func(block)
                )

        markdown_info["page_index"] = self["page_index"]
        markdown_info["input_path"] = self["input_path"]
        markdown_info["markdown_texts"] = markdown_content
        for img in self["imgs_in_doc"]:
            markdown_info["markdown_images"][img["path"]] = img["img"]

        return markdown_info



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from typing import Any, Dict, List

from paddlex_hps_server import BaseTritonPythonModel, schemas, utils


class TritonPythonModel(BaseTritonPythonModel):
    def get_input_model_type(self):
        return schemas.m_3d_bev_detection.InferRequest

    def get_result_model_type(self):
        return schemas.m_3d_bev_detection.InferResult

    def run(self, input, log_id):
        file_bytes = utils.get_raw_bytes(input.tar)
        tar_path = utils.write_to_temp_file(
            file_bytes,
            suffix=".tar",
        )

        try:
            result = list(
                self.pipeline(
                    tar_path,
                )
            )[0]
        finally:
            os.unlink(tar_path)

        objects: List[Dict[str, Any]] = []
        for box, label, score in zip(
            result["boxes_3d"], result["labels_3d"], result["scores_3d"]
        ):
            objects.append(
                dict(
                    bbox=box,
                    categoryId=label,
                    score=score,
                )
            )

        return schemas.m_3d_bev_detection.InferResult(detectedObjects=objects)



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file is based on https://github.com/Kwai-Keye/Keye/blob/main/keye-vl-8b-preview/modeling_keye.py
# Original header:
# Copyright 2025 The Keye Team and The HuggingFace Inc. team. All rights reserved.
#
# This code is based on EleutherAI's GPT-NeoX library and the GPT-NeoX
# and OPT implementations in this library. It has been modified from its
# original forms to accommodate minor architectural differences compared
# to GPT-NeoX and OPT used by the Meta AI team that trained the model.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import math

import paddle
import paddle.nn as nn


class GELUActivation(nn.Layer):
    """
    Original Implementation of the GELU activation function in Google BERT repo when initially created. For
    information: OpenAI GPT's GELU is slightly different (and gives slightly different results): 0.5 * x * (1 +
    torch.tanh(math.sqrt(2 / math.pi) * (x + 0.044715 * torch.pow(x, 3)))) This is now written in C in nn.functional
    Also see the Gaussian Error Linear Units paper: https://arxiv.org/abs/1606.08415
    """

    def __init__(self, use_gelu_python: bool = False):
        super().__init__()
        if use_gelu_python:
            self.act = self._gelu_python
        else:
            self.act = nn.functional.gelu

    def _gelu_python(self, input):
        return input * 0.5 * (1.0 + paddle.erf(input / math.sqrt(2.0)))

    def forward(self, input):
        return self.act(input)


class Projector(nn.Layer):

    def __init__(self, text_config, vision_config):
        super().__init__()
        self.text_config = text_config
        self.vision_config = vision_config
        self.merge_kernel_size = (2, 2)

        self.hidden_size = (
            self.vision_config.hidden_size
            * self.merge_kernel_size[0]
            * self.merge_kernel_size[1]
        )

        self.pre_norm = nn.LayerNorm(self.vision_config.hidden_size, epsilon=1e-05)
        self.linear_1 = nn.Linear(self.hidden_size, self.hidden_size)
        self.act = GELUActivation()
        self.linear_2 = nn.Linear(self.hidden_size, self.text_config.hidden_size)

    def forward(self, image_features, image_grid_thw):
        m1, m2 = self.merge_kernel_size
        if isinstance(image_features, (list, tuple)):
            processed_features = list()
            for image_feature, image_grid in zip(image_features, image_grid_thw):
                image_feature = self.pre_norm(image_feature)  # shape: (T*H*W, D)
                t, h, w = image_grid
                from einops import rearrange

                image_feature = rearrange(
                    image_feature,
                    "(t h p1 w p2) d -> (t h w) (p1 p2 d)",
                    t=int(t),
                    h=int(h // m1),
                    p1=int(m1),
                    w=int(w // m2),
                    p2=int(m2),
                )
                hidden_states = self.linear_1(image_feature)
                hidden_states = self.act(hidden_states)
                hidden_states = self.linear_2(hidden_states)
                processed_features.append(hidden_states)

            return processed_features

        dims = image_features.shape[:-1]
        dim = image_features.shape[-1]
        image_features = paddle.reshape(image_features, [-1, dim])
        hidden_states = self.pre_norm(image_features)
        hidden_states = paddle.reshape(hidden_states, [-1, self.hidden_size])
        hidden_states = self.linear_1(hidden_states)
        hidden_states = self.act(hidden_states)
        hidden_states = self.linear_2(hidden_states)
        return paddle.reshape(hidden_states, [*dims, -1])



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import platform
import sys


def process_paddle_lite(paddle_lite_so_path):
    if platform.system().lower() != "linux":
        return
    rpaths = ["$ORIGIN", "$ORIGIN/mklml/lib/"]
    patchelf_exe = os.getenv("PATCHELF_EXE", "patchelf")
    for root, dirs, files in os.walk(paddle_lite_so_path):
        for lib in files:
            if ".so" in lib:
                paddle_lite_so_file = os.path.join(root, lib)
                command = "{} --set-rpath '{}' {}".format(
                    patchelf_exe, ":".join(rpaths), paddle_lite_so_file
                )
                if platform.machine() != "sw_64" and platform.machine() != "mips64":
                    assert (
                        os.system(command) == 0
                    ), "patchelf {} failed, the command: {}".format(
                        paddle_lite_so_file, command
                    )


if __name__ == "__main__":
    process_paddle_lite(sys.argv[1])



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from ..... import UltraInferModel, ModelFormat
from ..... import c_lib_wrap as C


class AdaFacePreprocessor:
    def __init__(self):
        """Create a preprocessor for AdaFace Model"""
        self._preprocessor = C.vision.faceid.AdaFacePreprocessor()

    def run(self, input_ims):
        """Preprocess input images for AdaFace Model

        :param: input_ims: (list of numpy.ndarray)The input image
        :return: list of FDTensor, include image, scale_factor, im_shape
        """
        return self._preprocessor.run(input_ims)


class AdaFacePostprocessor:
    def __init__(self):
        """Create a postprocessor for AdaFace Model"""
        self._postprocessor = C.vision.faceid.AdaFacePostprocessor()

    def run(self, runtime_results):
        """Postprocess the runtime results for PaddleClas Model

        :param: runtime_results: (list of FDTensor)The output FDTensor results from runtime
        :return: list of FaceRecognitionResult(If the runtime_results is predict by batched samples, the length of this list equals to the batch size)
        """
        return self._postprocessor.run(runtime_results)

    @property
    def l2_normalize(self):
        """
        confidence threshold for postprocessing, default is 0.5
        """
        return self._postprocessor.l2_normalize


class AdaFace(UltraInferModel):
    def __init__(
        self,
        model_file,
        params_file="",
        runtime_option=None,
        model_format=ModelFormat.ONNX,
    ):
        """Load a AdaFace model exported by PaddleClas.

        :param model_file: (str)Path of model file, e.g adaface/model.pdmodel
        :param params_file: (str)Path of parameters file, e.g adaface/model.pdiparams, if the model_fomat is ModelFormat.ONNX, this param will be ignored, can be set as empty string
        :param runtime_option: (ultra_infer.RuntimeOption)RuntimeOption for inference this model, if it's None, will use the default backend on CPU
        :param model_format: (ultra_infer.ModelForamt)Model format of the loaded model
        """
        super(AdaFace, self).__init__(runtime_option)
        self._model = C.vision.faceid.AdaFace(
            model_file, params_file, self._runtime_option, model_format
        )
        assert self.initialized, "AdaFace model initialize failed."

    def predict(self, im):
        """Detect an input image

        :param im: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
        :return: DetectionResult
        """

        assert im is not None, "The input image data is None."
        return self._model.predict(im)

    def batch_predict(self, images):
        """Detect a batch of input image list

        :param im: (list of numpy.ndarray) The input image list, each element is a 3-D array with layout HWC, BGR format
        :return list of DetectionResult
        """

        return self._model.batch_predict(images)

    @property
    def preprocessor(self):
        """Get AdaFacePreprocessor object of the loaded model

        :return AdaFacePreprocessor
        """
        return self._model.preprocessor

    @property
    def postprocessor(self):
        """Get AdaFacePostprocessor object of the loaded model

        :return AdaFacePostprocessor
        """
        return self._model.postprocessor



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any, Dict, Final, List, Tuple

from paddlex_hps_server import (
    BaseTritonPythonModel,
    app_common,
    protocol,
    schemas,
    utils,
)
from paddlex_hps_server.storage import SupportsGetURL, create_storage

_DEFAULT_MAX_NUM_INPUT_IMGS: Final[int] = 10
_DEFAULT_MAX_OUTPUT_IMG_SIZE: Final[Tuple[int, int]] = (2000, 2000)


class TritonPythonModel(BaseTritonPythonModel):
    def initialize(self, args):
        super().initialize(args)
        self.context = {}
        self.context["file_storage"] = None
        self.context["return_img_urls"] = False
        self.context["max_num_input_imgs"] = _DEFAULT_MAX_NUM_INPUT_IMGS
        self.context["max_output_img_size"] = _DEFAULT_MAX_OUTPUT_IMG_SIZE
        if self.app_config.extra:
            if "file_storage" in self.app_config.extra:
                self.context["file_storage"] = create_storage(
                    self.app_config.extra["file_storage"]
                )
            if "return_img_urls" in self.app_config.extra:
                self.context["return_img_urls"] = self.app_config.extra[
                    "return_img_urls"
                ]
            if "max_num_input_imgs" in self.app_config.extra:
                self.context["max_num_input_imgs"] = self.app_config.extra[
                    "max_num_input_imgs"
                ]
            if "max_output_img_size" in self.app_config.extra:
                self.context["max_output_img_size"] = self.app_config.extra[
                    "max_output_img_size"
                ]
        if self.context["return_img_urls"]:
            file_storage = self.context["file_storage"]
            if not file_storage:
                raise ValueError(
                    "The file storage must be properly configured when URLs need to be returned."
                )
            if not isinstance(file_storage, SupportsGetURL):
                raise TypeError(f"{type(file_storage)} does not support getting URLs.")

    def get_input_model_type(self):
        return schemas.pp_chatocrv3_doc.AnalyzeImagesRequest

    def get_result_model_type(self):
        return schemas.pp_chatocrv3_doc.AnalyzeImagesResult

    def run(self, input, log_id):
        if input.fileType is None:
            if utils.is_url(input.file):
                maybe_file_type = utils.infer_file_type(input.file)
                if maybe_file_type is None or not (
                    maybe_file_type == "PDF" or maybe_file_type == "IMAGE"
                ):
                    return protocol.create_aistudio_output_without_result(
                        422,
                        "Unsupported file type",
                        log_id=log_id,
                    )
                file_type = maybe_file_type
            else:
                return protocol.create_aistudio_output_without_result(
                    422,
                    "File type cannot be determined",
                    log_id=log_id,
                )
        else:
            file_type = "PDF" if input.fileType == 0 else "IMAGE"
        visualize_enabled = (
            input.visualize
            if input.visualize is not None
            else self.app_config.visualize
        )

        file_bytes = utils.get_raw_bytes(input.file)
        images, data_info = utils.file_to_images(
            file_bytes,
            file_type,
            max_num_imgs=self.context["max_num_input_imgs"],
        )

        result = self.pipeline.visual_predict(
            images,
            use_doc_orientation_classify=input.useDocOrientationClassify,
            use_doc_unwarping=input.useDocUnwarping,
            use_seal_recognition=input.useSealRecognition,
            use_table_recognition=input.useTableRecognition,
            layout_threshold=input.layoutThreshold,
            layout_nms=input.layoutNms,
            layout_unclip_ratio=input.layoutUnclipRatio,
            layout_merge_bboxes_mode=input.layoutMergeBboxesMode,
            text_det_limit_side_len=input.textDetLimitSideLen,
            text_det_limit_type=input.textDetLimitType,
            text_det_thresh=input.textDetThresh,
            text_det_box_thresh=input.textDetBoxThresh,
            text_det_unclip_ratio=input.textDetUnclipRatio,
            text_rec_score_thresh=input.textRecScoreThresh,
            seal_det_limit_side_len=input.sealDetLimitSideLen,
            seal_det_limit_type=input.sealDetLimitType,
            seal_det_thresh=input.sealDetThresh,
            seal_det_box_thresh=input.sealDetBoxThresh,
            seal_det_unclip_ratio=input.sealDetUnclipRatio,
            seal_rec_score_thresh=input.sealRecScoreThresh,
        )

        layout_parsing_results: List[Dict[str, Any]] = []
        visual_info: List[dict] = []
        for i, (img, item) in enumerate(zip(images, result)):
            pruned_res = app_common.prune_result(
                item["layout_parsing_result"].json["res"]
            )
            if visualize_enabled:
                imgs = {
                    "input_img": img,
                    **item["layout_parsing_result"].img,
                }
                imgs = app_common.postprocess_images(
                    imgs,
                    log_id,
                    filename_template=f"{{key}}_{i}.jpg",
                    file_storage=self.context["file_storage"],
                    return_urls=self.context["return_img_urls"],
                    max_img_size=self.context["max_output_img_size"],
                )
            else:
                imgs = {}
            layout_parsing_results.append(
                dict(
                    prunedResult=pruned_res,
                    outputImages=(
                        {k: v for k, v in imgs.items() if k != "input_img"}
                        if imgs
                        else None
                    ),
                    inputImage=imgs.get("input_img"),
                )
            )
            visual_info.append(item["visual_info"])

        return schemas.pp_chatocrv3_doc.AnalyzeImagesResult(
            layoutParsingResults=layout_parsing_results,
            visualInfo=visual_info,
            dataInfo=data_info,
        )



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from ... import c_lib_wrap as C


class PPTinyPose(object):
    def __init__(self, det_model=None, pptinypose_model=None):
        """Set initialized detection model object and pptinypose model object

        :param det_model: (ultra_infer.vision.detection.PicoDet)Initialized detection model object
        :param pptinypose_model: (ultra_infer.vision.keypointdetection.PPTinyPose)Initialized pptinypose model object
        """
        assert (
            det_model is not None or pptinypose_model is not None
        ), "The det_model and pptinypose_model cannot be None."
        self._pipeline = C.pipeline.PPTinyPose(
            det_model._model, pptinypose_model._model
        )

    def predict(self, input_image):
        """Predict the keypoint detection result for an input image

        :param im: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
        :return: KeyPointDetectionResult
        """
        return self._pipeline.predict(input_image)

    @property
    def detection_model_score_threshold(self):
        """Attribute of PPTinyPose pipeline model. Stating the score threshold for detecting model to filter bbox before inputting pptinypose model

        :return: value of detection_model_score_threshold(float)
        """
        return self._pipeline.detection_model_score_threshold

    @detection_model_score_threshold.setter
    def detection_model_score_threshold(self, value):
        """Set attribute detection_model_score_threshold of PPTinyPose pipeline model.

        :param value: (float)The value to set use_dark
        """
        assert isinstance(
            value, float
        ), "The value to set `detection_model_score_threshold` must be type of float."
        self._pipeline.detection_model_score_threshold = value



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List, Optional, Union, Dict

import numpy as np

from ..base import PyOnlyProcessor

__all__ = [
    "CutOff",
    "Normalize",
    "Denormalize",
    "BuildTSDataset",
    "CalcTimeFeatures",
    "BuildPaddedMask",
    "DataFrame2Arrays",
]

_MAX_WINDOW = 183 + 17


def _cal_year(
    x: np.datetime64,
):
    return x.year


def _cal_month(
    x: np.datetime64,
):
    return x.month


def _cal_day(
    x: np.datetime64,
):
    return x.day


def _cal_hour(
    x: np.datetime64,
):
    return x.hour


def _cal_weekday(
    x: np.datetime64,
):
    return x.dayofweek


def _cal_quarter(
    x: np.datetime64,
):
    return x.quarter


def _cal_hourofday(
    x: np.datetime64,
):
    return x.hour / 23.0 - 0.5


def _cal_dayofweek(
    x: np.datetime64,
):
    return x.dayofweek / 6.0 - 0.5


def _cal_dayofmonth(
    x: np.datetime64,
):
    return x.day / 30.0 - 0.5


def _cal_dayofyear(
    x: np.datetime64,
):
    return x.dayofyear / 364.0 - 0.5


def _cal_weekofyear(
    x: np.datetime64,
):
    return x.weekofyear / 51.0 - 0.5


def _cal_holiday(
    x: np.datetime64,
):
    import chinese_calendar

    return float(chinese_calendar.is_holiday(x))


def _cal_workday(
    x: np.datetime64,
):
    import chinese_calendar

    return float(chinese_calendar.is_workday(x))


def _cal_minuteofhour(
    x: np.datetime64,
):
    return x.minute / 59 - 0.5


def _cal_monthofyear(
    x: np.datetime64,
):
    return x.month / 11.0 - 0.5


_CAL_DATE_METHOD = {
    "year": _cal_year,
    "month": _cal_month,
    "day": _cal_day,
    "hour": _cal_hour,
    "weekday": _cal_weekday,
    "quarter": _cal_quarter,
    "minuteofhour": _cal_minuteofhour,
    "monthofyear": _cal_monthofyear,
    "hourofday": _cal_hourofday,
    "dayofweek": _cal_dayofweek,
    "dayofmonth": _cal_dayofmonth,
    "dayofyear": _cal_dayofyear,
    "weekofyear": _cal_weekofyear,
    "is_holiday": _cal_holiday,
    "is_workday": _cal_workday,
}


def _load_from_one_dataframe(
    data: Union["pd.DataFrame", "pd.Series"],  # noqa: F821
    time_col: Optional[str] = None,
    value_cols: Optional[Union[List[str], str]] = None,
    freq: Optional[Union[str, int]] = None,
    drop_tail_nan: bool = False,
    dtype: Optional[Union[type, Dict[str, type]]] = None,
):
    import pandas as pd

    series_data = None
    if value_cols is None:
        if isinstance(data, pd.Series):
            series_data = data.copy()
        else:
            series_data = data.loc[:, data.columns != time_col].copy()
    else:
        series_data = data.loc[:, value_cols].copy()

    if time_col:
        if time_col not in data.columns:
            raise ValueError(
                "The time column: {} doesn't exist in the `data`!".format(time_col)
            )
        time_col_vals = data.loc[:, time_col]
    else:
        time_col_vals = data.index

    if np.issubdtype(time_col_vals.dtype, np.integer) and isinstance(freq, str):
        time_col_vals = time_col_vals.astype(str)

    if np.issubdtype(time_col_vals.dtype, np.integer):
        if freq:
            if not isinstance(freq, int) or freq < 1:
                raise ValueError(
                    "The type of `freq` should be `int` when the type of `time_col` is `RangeIndex`."
                )
        else:
            freq = 1
        start_idx, stop_idx = min(time_col_vals), max(time_col_vals) + freq
        if (stop_idx - start_idx) / freq != len(data):
            raise ValueError("The number of rows doesn't match with the RangeIndex!")
        time_index = pd.RangeIndex(start=start_idx, stop=stop_idx, step=freq)
    elif np.issubdtype(time_col_vals.dtype, np.object_) or np.issubdtype(
        time_col_vals.dtype, np.datetime64
    ):
        time_col_vals = pd.to_datetime(time_col_vals, infer_datetime_format=True)
        time_index = pd.DatetimeIndex(time_col_vals)
        if freq:
            if not isinstance(freq, str):
                raise ValueError(
                    "The type of `freq` should be `str` when the type of `time_col` is `DatetimeIndex`."
                )
        else:
            # If freq is not provided and automatic inference fail, throw exception
            freq = pd.infer_freq(time_index)
            if freq is None:
                raise ValueError(
                    "Failed to infer the `freq`. A valid `freq` is required."
                )
            if freq[0] == "-":
                freq = freq[1:]
    else:
        raise ValueError("The type of `time_col` is invalid.")
    if isinstance(series_data, pd.Series):
        series_data = series_data.to_frame()
    series_data.set_index(time_index, inplace=True)
    series_data.sort_index(inplace=True)
    return series_data


def _load_from_dataframe(
    df: "pd.DataFrame",  # noqa: F821
    group_id: str = None,
    time_col: Optional[str] = None,
    target_cols: Optional[Union[List[str], str]] = None,
    label_col: Optional[Union[List[str], str]] = None,
    observed_cov_cols: Optional[Union[List[str], str]] = None,
    feature_cols: Optional[Union[List[str], str]] = None,
    known_cov_cols: Optional[Union[List[str], str]] = None,
    static_cov_cols: Optional[Union[List[str], str]] = None,
    freq: Optional[Union[str, int]] = None,
    fill_missing_dates: bool = False,
    fillna_method: str = "pre",
    fillna_window_size: int = 10,
    **kwargs,
):
    dfs = []  # separate multiple group
    if group_id is not None:
        group_unique = df[group_id].unique()
        for column in group_unique:
            dfs.append(df[df[group_id].isin([column])])
    else:
        dfs = [df]
    res = []
    if label_col:
        if isinstance(label_col, str) and len(label_col) > 1:
            raise ValueError("The length of label_col must be 1.")
        target_cols = label_col
    if feature_cols:
        observed_cov_cols = feature_cols
    for df in dfs:
        target = None
        observed_cov = None
        known_cov = None
        static_cov = dict()
        if not any([target_cols, observed_cov_cols, known_cov_cols, static_cov_cols]):
            target = _load_from_one_dataframe(
                df,
                time_col,
                [a for a in df.columns if a != time_col],
                freq,
            )

        else:
            if target_cols:
                target = _load_from_one_dataframe(
                    df,
                    time_col,
                    target_cols,
                    freq,
                )

            if observed_cov_cols:
                observed_cov = _load_from_one_dataframe(
                    df,
                    time_col,
                    observed_cov_cols,
                    freq,
                )

            if known_cov_cols:
                known_cov = _load_from_one_dataframe(
                    df,
                    time_col,
                    known_cov_cols,
                    freq,
                )

            if static_cov_cols:
                if isinstance(static_cov_cols, str):
                    static_cov_cols = [static_cov_cols]
                for col in static_cov_cols:
                    if col not in df.columns or len(np.unique(df[col])) != 1:
                        raise ValueError(
                            "static cov cals data is not in columns or schema is not right!"
                        )
                    static_cov[col] = df[col].iloc[0]
        res.append(
            {
                "past_target": target,
                "observed_cov_numeric": observed_cov,
                "known_cov_numeric": known_cov,
                "static_cov_numeric": static_cov,
            }
        )
    return res[0]


def _distance_to_holiday(holiday):
    def _distance_to_day(index):
        import pandas as pd

        holiday_date = holiday.dates(
            index - pd.Timedelta(days=_MAX_WINDOW),
            index + pd.Timedelta(days=_MAX_WINDOW),
        )
        assert (
            len(holiday_date) != 0
        ), f"No closest holiday for the date index {index} found."
        # It sometimes returns two dates if it is exactly half a year after the
        # holiday. In this case, the smaller distance (182 days) is returned.
        return float((index - holiday_date[0]).days)

    return _distance_to_day


def _to_time_features(
    dataset, freq, feature_cols, extend_points, inplace: bool = False
):
    import pandas as pd

    new_ts = dataset
    if not inplace:
        new_ts = dataset.copy()
    # Get known_cov
    kcov = new_ts["known_cov_numeric"]
    if not kcov:
        tf_kcov = new_ts["past_target"].index.to_frame()
    else:
        tf_kcov = kcov.index.to_frame()
    time_col = tf_kcov.columns[0]
    if np.issubdtype(tf_kcov[time_col].dtype, np.integer):
        raise ValueError(
            "The time_col can't be the type of numpy.integer, and it must be the type of numpy.datetime64"
        )
    if not kcov:
        freq = freq if freq is not None else pd.infer_freq(tf_kcov[time_col])
        extend_time = pd.date_range(
            start=tf_kcov[time_col][-1],
            freq=freq,
            periods=extend_points + 1,
            closed="right",
            name=time_col,
        ).to_frame()
        tf_kcov = pd.concat([tf_kcov, extend_time])

    for k in feature_cols:
        if k != "holidays":
            v = tf_kcov[time_col].apply(lambda x: _CAL_DATE_METHOD[k](x))
            v.index = tf_kcov[time_col]

            if new_ts["known_cov_numeric"] is None:
                new_ts["known_cov_numeric"] = pd.DataFrame(v.rename(k), index=v.index)
            else:
                new_ts["known_cov_numeric"][k] = v.rename(k).reindex(
                    new_ts["known_cov_numeric"].index
                )

        else:
            from pandas.tseries.offsets import DateOffset, Easter, Day
            from pandas.tseries import holiday as hd
            from sklearn.preprocessing import StandardScaler

            _EASTER_SUNDAY = hd.Holiday(
                "Easter Sunday", month=1, day=1, offset=[Easter(), Day(0)]
            )
            _NEW_YEARS_DAY = hd.Holiday("New Years Day", month=1, day=1)
            _SUPER_BOWL = hd.Holiday(
                "Superbowl", month=2, day=1, offset=DateOffset(weekday=hd.SU(1))
            )
            _MOTHERS_DAY = hd.Holiday(
                "Mothers Day", month=5, day=1, offset=DateOffset(weekday=hd.SU(2))
            )
            _INDEPENDENCE_DAY = hd.Holiday("Independence Day", month=7, day=4)
            _CHRISTMAS_EVE = hd.Holiday("Christmas", month=12, day=24)
            _CHRISTMAS_DAY = hd.Holiday("Christmas", month=12, day=25)
            _NEW_YEARS_EVE = hd.Holiday("New Years Eve", month=12, day=31)
            _BLACK_FRIDAY = hd.Holiday(
                "Black Friday",
                month=11,
                day=1,
                offset=[pd.DateOffset(weekday=hd.TH(4)), Day(1)],
            )
            _CYBER_MONDAY = hd.Holiday(
                "Cyber Monday",
                month=11,
                day=1,
                offset=[pd.DateOffset(weekday=hd.TH(4)), Day(4)],
            )

            _HOLYDAYS = [
                hd.EasterMonday,
                hd.GoodFriday,
                hd.USColumbusDay,
                hd.USLaborDay,
                hd.USMartinLutherKingJr,
                hd.USMemorialDay,
                hd.USPresidentsDay,
                hd.USThanksgivingDay,
                _EASTER_SUNDAY,
                _NEW_YEARS_DAY,
                _SUPER_BOWL,
                _MOTHERS_DAY,
                _INDEPENDENCE_DAY,
                _CHRISTMAS_EVE,
                _CHRISTMAS_DAY,
                _NEW_YEARS_EVE,
                _BLACK_FRIDAY,
                _CYBER_MONDAY,
            ]
            holidays_col = []
            for i, H in enumerate(_HOLYDAYS):
                v = tf_kcov[time_col].apply(_distance_to_holiday(H))
                v.index = tf_kcov[time_col]
                holidays_col.append(k + "_" + str(i))
                if new_ts["known_cov_numeric"] is None:
                    new_ts["known_cov_numeric"] = pd.DataFrame(
                        v.rename(k + "_" + str(i)), index=v.index
                    )
                else:
                    new_ts["known_cov_numeric"][k + "_" + str(i)] = v.rename(k).reindex(
                        new_ts["known_cov_numeric"].index
                    )

            scaler = StandardScaler()
            scaler.fit(new_ts["known_cov_numeric"][holidays_col])
            new_ts["known_cov_numeric"][holidays_col] = scaler.transform(
                new_ts["known_cov_numeric"][holidays_col]
            )
    return new_ts


class CutOff(PyOnlyProcessor):
    def __init__(self, size):
        super().__init__()
        self._size = size

    def __call__(self, data):
        ts = data["ts"]
        ori_ts = data["ori_ts"]

        skip_len = self._size.get("skip_chunk_len", 0)
        if len(ts) < self._size["in_chunk_len"] + skip_len:
            raise ValueError(
                f"The length of the input data is {len(ts)}, but it should be at least {self._size['in_chunk_len'] + self._size['skip_chunk_len']} for training."
            )
        ts_data = ts[-(self._size["in_chunk_len"] + skip_len) :]

        return {**data, "ts": ts_data, "ori_ts": ts_data}


class Normalize(PyOnlyProcessor):
    def __init__(self, scale_path, params_info):
        import joblib

        super().__init__()
        self._scaler = joblib.load(scale_path)
        self._params_info = params_info

    def __call__(self, data):
        ts = data["ts"]

        if self._params_info.get("target_cols", None) is not None:
            ts[self._params_info["target_cols"]] = self._scaler.transform(
                ts[self._params_info["target_cols"]]
            )
        if self._params_info.get("feature_cols", None) is not None:
            ts[self._params_info["feature_cols"]] = self._scaler.transform(
                ts[self._params_info["feature_cols"]]
            )

        return {**data, "ts": ts}


class Denormalize(PyOnlyProcessor):
    def __init__(self, scale_path, params_info):
        import joblib

        super().__init__()
        self._scaler = joblib.load(scale_path)
        self._params_info = params_info

    def __call__(self, data):
        pred = data["pred"]

        scale_cols = pred.columns.values.tolist()
        pred[scale_cols] = self._scaler.inverse_transform(pred[scale_cols])

        return {**data, "pred": pred}


class BuildTSDataset(PyOnlyProcessor):
    def __init__(self, params_info):
        super().__init__()
        self._params_info = params_info

    def __call__(self, data):
        ts = data["ts"]
        ori_ts = data["ori_ts"]

        ts_data = _load_from_dataframe(ts, **self._params_info)

        return {**data, "ts": ts_data, "ori_ts": ts_data}


class CalcTimeFeatures(PyOnlyProcessor):
    def __init__(self, params_info, size, holiday=False):
        super().__init__()
        self._freq = params_info["freq"]
        self._size = size
        self._holiday = holiday

    def __call__(self, data):
        ts = data["ts"]

        if not self._holiday:
            ts = _to_time_features(
                ts,
                self._freq,
                ["hourofday", "dayofmonth", "dayofweek", "dayofyear"],
                self._size["out_chunk_len"],
            )
        else:
            ts = _to_time_features(
                ts,
                self._freq,
                [
                    "minuteofhour",
                    "hourofday",
                    "dayofmonth",
                    "dayofweek",
                    "dayofyear",
                    "monthofyear",
                    "weekofyear",
                    "holidays",
                ],
                self._size["out_chunk_len"],
            )

        return {**data, "ts": ts}


class BuildPaddedMask(PyOnlyProcessor):
    def __init__(self, input_data):
        super().__init__()
        self._input_data = input_data

    def __call__(self, data):
        ts = data["ts"]

        if "features" in self._input_data:
            ts["features"] = ts["past_target"]

        if "pad_mask" in self._input_data:
            target_dim = len(ts["features"])
            max_length = self._input_data["pad_mask"][-1]
            if max_length > 0:
                ones = np.ones(max_length, dtype=np.int32)
                if max_length != target_dim:
                    target_ndarray = np.array(ts["features"]).astype(np.float32)
                    target_ndarray_final = np.zeros(
                        [max_length, target_dim], dtype=np.int32
                    )
                    end = min(target_dim, max_length)
                    target_ndarray_final[:end, :] = target_ndarray
                    ts["features"] = target_ndarray_final
                    ones[end:] = 0.0
                    ts["pad_mask"] = ones
                else:
                    ts["pad_mask"] = ones

        return {**data, "ts": ts}


class DataFrame2Arrays(PyOnlyProcessor):
    def __init__(self, input_data):
        super().__init__()
        self._input_data = input_data

    def __call__(self, data):
        ts = data["ts"]

        ts_list = []
        input_name = list(self._input_data.keys())
        input_name.sort()
        for key in input_name:
            ts_list.append(np.array(ts[key]).astype("float32"))

        return {**data, "ts": ts_list}



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file is based on https://github.com/Kwai-Keye/Keye/blob/main/keye-vl-8b-preview/modeling_keye.py
# Original header:
# Copyright 2025 The Keye Team and The HuggingFace Inc. team. All rights reserved.
#
# This code is based on EleutherAI's GPT-NeoX library and the GPT-NeoX
# and OPT implementations in this library. It has been modified from its
# original forms to accommodate minor architectural differences compared
# to GPT-NeoX and OPT used by the Meta AI team that trained the model.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from contextvars import ContextVar
from dataclasses import dataclass
from typing import List, Optional, Tuple, Union

import numpy as np
import paddle
import paddle.nn as nn

from ....common.vlm.generation import GenerationMixin
from ....common.vlm.transformers.model_outputs import (
    CausalLMOutputWithCrossAttentions,
    ModelOutput,
)
from ._config import PaddleOCRVLConfig
from ._ernie import Ernie4_5Model, Ernie4_5PretrainedModel
from ._projector import Projector
from ._siglip import SiglipVisionModel


@dataclass
class PaddleOCRVLCausalLMOutputWithPast(ModelOutput):
    loss: Optional[paddle.Tensor] = None
    logits: paddle.Tensor = None
    past_key_values: Optional[List[paddle.Tensor]] = None
    hidden_states: Optional[Tuple[paddle.Tensor]] = None
    attentions: Optional[Tuple[paddle.Tensor]] = None
    rope_deltas: Optional[paddle.Tensor] = None


class PaddleOCRVLForConditionalGeneration(Ernie4_5PretrainedModel, GenerationMixin):
    _tied_weights_keys = ["lm_head.weight"]
    config_class = PaddleOCRVLConfig
    _no_split_modules = ["Ernie4_5DecoderLayer", "SiglipEncoderLayer"]

    base_model_prefix = ""

    def __init__(self, config):
        super().__init__(config)

        self.mlp_AR = Projector(config, config.vision_config)
        self.visual = SiglipVisionModel(config.vision_config)
        self.model = Ernie4_5Model(config)
        self.vocab_size = config.vocab_size
        self.lm_head = nn.Linear(config.hidden_size, config.vocab_size, bias_attr=False)
        self.rope_deltas_var = ContextVar("rope_deltas", default=None)

    def get_input_embeddings(self):
        return self.model.embed_tokens

    def set_input_embeddings(self, value):
        self.model.embed_tokens = value

    def get_output_embeddings(self):
        return self.lm_head

    def set_output_embeddings(self, new_embeddings):
        self.lm_head = new_embeddings

    def set_decoder(self, decoder):
        self.model = decoder

    def get_decoder(self):
        return self.model

    def get_rope_index(
        self,
        input_ids: Optional[paddle.Tensor] = None,
        image_grid_thw: Optional[paddle.Tensor] = None,
        video_grid_thw: Optional[paddle.Tensor] = None,
        second_per_grid_ts: Optional[paddle.Tensor] = None,
        attention_mask: Optional[paddle.Tensor] = None,
    ) -> Tuple[paddle.Tensor, paddle.Tensor]:
        """
        Calculate the 3D rope index based on image and video's temporal, height and width in LLM.

        Explanation:
            Each embedding sequence contains vision embedding and text embedding or just contains text embedding.

            For pure text embedding sequence, the rotary position embedding has no difference with modern LLMs.
            Examples:
                input_ids: [T T T T T], here T is for text.
                temporal position_ids: [0, 1, 2, 3, 4]
                height position_ids: [0, 1, 2, 3, 4]
                width position_ids: [0, 1, 2, 3, 4]

            For vision and text embedding sequence, we calculate 3D rotary position embedding for vision part
            and 1D rotary position embedding for text part.
            Examples:
                Temporal (Time): 3 patches, representing different segments of the video in time.
                Height: 2 patches, dividing each frame vertically.
                Width: 2 patches, dividing each frame horizontally.
                We also have some important parameters:
                fps (Frames Per Second): The video's frame rate, set to 1. This means one frame is processed each second.
                tokens_per_second: This is a crucial parameter. It dictates how many "time-steps" or "temporal tokens" are conceptually packed into a one-second interval of the video. In this case, we have 25 tokens per second. So each second of the video will be represented with 25 separate time points. It essentially defines the temporal granularity.
                temporal_patch_size: The number of frames that compose one temporal patch. Here, it's 2 frames.
                interval: The step size for the temporal position IDs, calculated as tokens_per_second * temporal_patch_size / fps. In this case, 25 * 2 / 1 = 50. This means that each temporal patch will be have a difference of 50 in the temporal position IDs.
                input_ids: [V V V V V V V V V V V V T T T T T], here V is for vision.
                vision temporal position_ids: [0, 0, 0, 0, 50, 50, 50, 50, 100, 100, 100, 100]
                vision height position_ids: [0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1]
                vision width position_ids: [0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1]
                text temporal position_ids: [101, 102, 103, 104, 105]
                text height position_ids: [101, 102, 103, 104, 105]
                text width position_ids: [101, 102, 103, 104, 105]
                Here we calculate the text start position_ids as the max vision position_ids plus 1.

        Args:
            input_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`):
                Indices of input sequence tokens in the vocabulary. Padding will be ignored by default should you provide
                it.
            image_grid_thw (`paddle.Tensor` of shape `(num_images, 3)`, *optional*):
                The temporal, height and width of feature shape of each image in LLM.
            video_grid_thw (`paddle.Tensor` of shape `(num_videos, 3)`, *optional*):
                The temporal, height and width of feature shape of each video in LLM.
            second_per_grid_ts (`paddle.Tensor` of shape `(num_videos)`, *optional*):
                The time interval (in seconds) for each grid along the temporal dimension in the 3D position IDs.
            attention_mask (`paddle.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
                Mask to avoid performing attention on padding token indices. Mask values selected in `[0, 1]`:

                - 1 for tokens that are **not masked**,
                - 0 for tokens that are **masked**.

        Returns:
            position_ids (`paddle.Tensor` of shape `(3, batch_size, sequence_length)`)
            mrope_position_deltas (`paddle.Tensor` of shape `(batch_size)`)
        """
        spatial_merge_size = self.config.vision_config.spatial_merge_size
        image_token_id = self.config.image_token_id
        video_token_id = self.config.video_token_id
        vision_start_token_id = self.config.vision_start_token_id
        mrope_position_deltas = []
        if input_ids is not None and (
            image_grid_thw is not None or video_grid_thw is not None
        ):
            total_input_ids = input_ids
            if attention_mask is None:
                attention_mask = paddle.ones_like(total_input_ids)
            position_ids = paddle.ones(
                [3, input_ids.shape[0], input_ids.shape[1]],
                dtype=input_ids.dtype,
            )
            image_index, video_index = 0, 0
            for i, input_ids in enumerate(total_input_ids):
                input_ids = input_ids[attention_mask[i] == 1]
                image_nums, video_nums = 0, 0
                vision_start_indices = paddle.nonzero(
                    input_ids == vision_start_token_id
                ).squeeze(1)
                vision_tokens = input_ids[vision_start_indices + 1]
                image_nums = (vision_tokens == image_token_id).sum()
                video_nums = (vision_tokens == video_token_id).sum()
                input_tokens = input_ids.tolist()
                llm_pos_ids_list: list = []
                st = 0
                remain_images, remain_videos = image_nums, video_nums
                for _ in range(image_nums + video_nums):
                    if image_token_id in input_tokens and remain_images > 0:
                        ed_image = input_tokens.index(image_token_id, st)
                    else:
                        ed_image = len(input_tokens) + 1
                    if video_token_id in input_tokens and remain_videos > 0:
                        ed_video = input_tokens.index(video_token_id, st)
                    else:
                        ed_video = len(input_tokens) + 1
                    if ed_image < ed_video:
                        t, h, w = (
                            image_grid_thw[image_index][0],
                            image_grid_thw[image_index][1],
                            image_grid_thw[image_index][2],
                        )
                        second_per_grid_t = 0
                        image_index += 1
                        remain_images -= 1
                        ed = ed_image

                    else:
                        t, h, w = (
                            video_grid_thw[video_index][0],
                            video_grid_thw[video_index][1],
                            video_grid_thw[video_index][2],
                        )
                        if second_per_grid_ts is not None:
                            second_per_grid_t = second_per_grid_ts[video_index]
                        else:
                            second_per_grid_t = 1.0
                        video_index += 1
                        remain_videos -= 1
                        ed = ed_video
                    llm_grid_t, llm_grid_h, llm_grid_w = (
                        t.item(),
                        h.item() // spatial_merge_size,
                        w.item() // spatial_merge_size,
                    )
                    text_len = ed - st

                    st_idx = (
                        llm_pos_ids_list[-1].max() + 1
                        if len(llm_pos_ids_list) > 0
                        else 0
                    )
                    llm_pos_ids_list.append(
                        paddle.arange(text_len).reshape((1, -1)).expand((3, -1))
                        + st_idx
                    )

                    if paddle.is_tensor(second_per_grid_t):
                        second_per_grid_t = second_per_grid_t.detach().item()
                    range_tensor = paddle.arange(llm_grid_t).reshape((-1, 1))
                    expanded_range = range_tensor.expand((-1, llm_grid_h * llm_grid_w))

                    time_tensor = (
                        expanded_range
                        * second_per_grid_t
                        * self.config.vision_config.tokens_per_second
                    )

                    time_tensor_long = time_tensor.astype("int64")
                    t_index = time_tensor_long.flatten()

                    h_index = (
                        paddle.arange(llm_grid_h)
                        .reshape((1, -1, 1))
                        .expand((llm_grid_t, -1, llm_grid_w))
                        .flatten()
                    )
                    w_index = (
                        paddle.arange(llm_grid_w)
                        .reshape((1, 1, -1))
                        .expand((llm_grid_t, llm_grid_h, -1))
                        .flatten()
                    )
                    llm_pos_ids_list.append(
                        paddle.stack([t_index, h_index, w_index]) + text_len + st_idx
                    )
                    st = ed + llm_grid_t * llm_grid_h * llm_grid_w

                if st < len(input_tokens):
                    st_idx = (
                        llm_pos_ids_list[-1].max() + 1
                        if len(llm_pos_ids_list) > 0
                        else 0
                    )
                    text_len = len(input_tokens) - st
                    llm_pos_ids_list.append(
                        paddle.arange(text_len).reshape((1, -1)).expand((3, -1))
                        + st_idx
                    )

                llm_positions = paddle.concat(llm_pos_ids_list, axis=1).reshape((3, -1))
                position_ids[..., i, attention_mask[i] == 1] = llm_positions
                mrope_position_deltas.append(
                    llm_positions.max() + 1 - len(total_input_ids[i])
                )
            mrope_position_deltas = paddle.to_tensor(mrope_position_deltas).unsqueeze(1)
            return position_ids, mrope_position_deltas
        else:
            if attention_mask is not None:
                position_ids = attention_mask.long().cumsum(-1) - 1
                position_ids.masked_fill_(attention_mask == 0, 1)
                position_ids = position_ids.unsqueeze(0).expand((3, -1, -1))
                max_position_ids = position_ids.max(0, keepdim=False)[0].max(
                    -1, keepdim=True
                )[0]
                mrope_position_deltas = max_position_ids + 1 - attention_mask.shape[-1]
            else:
                position_ids = (
                    paddle.arange(input_ids.shape[1])
                    .reshape((1, 1, -1))
                    .expand((3, input_ids.shape[0], -1))
                )
                mrope_position_deltas = paddle.zeros(
                    [input_ids.shape[0], 1],
                    dtype=input_ids.dtype,
                )

            return position_ids, mrope_position_deltas

    def prepare_attention_mask_for_generation(
        self, input_ids, pad_token_id, eos_token_id
    ):
        """Avoid using attention_mask with flash_attn on generation."""
        if self.config.use_flash_attention:
            return None
        return super().prepare_attention_mask_for_generation(
            input_ids, pad_token_id, eos_token_id
        )

    def prepare_inputs_for_generation(
        self,
        input_ids,
        use_cache=False,
        past_key_values=None,
        inputs_embeds=None,
        pixel_values=None,
        pixel_values_videos=None,
        position_ids=None,
        **kwargs,
    ):
        if past_key_values:
            input_ids = input_ids[:, -1:]
            pixel_values = None
            pixel_values_videos = None

        # if `inputs_embeds` are passed, we only want to use them in the 1st generation step
        if inputs_embeds is not None and past_key_values is None:
            model_inputs = {"inputs_embeds": inputs_embeds}
        else:
            model_inputs = {"input_ids": input_ids}

        model_inputs.update(
            {
                "past_key_values": past_key_values,
                "use_cache": use_cache,
                "pixel_values": pixel_values,
                "pixel_values_videos": pixel_values_videos,
                "position_ids": None,
                **kwargs,
            }
        )

        return model_inputs

    def update_model_kwargs_for_generation(
        self, outputs, model_kwargs, is_encoder_decoder=False
    ):
        """
        Updates model kwargs for generation.

        Args:
            outputs (Any): Model outputs.
            model_kwargs (dict): Current model kwargs.
            is_encoder_decoder (bool): Whether using encoder-decoder architecture.

        Returns:
            dict: Updated model kwargs.
        """
        # update cache
        if (
            isinstance(outputs, tuple)
            and len(outputs) > 1
            and not isinstance(outputs[1], paddle.Tensor)
        ):
            model_kwargs["past_key_values"] = outputs[1]

        if (
            isinstance(outputs, CausalLMOutputWithCrossAttentions)
            and "past_key_values" in outputs
        ):
            model_kwargs["past_key_values"] = outputs.past_key_values

        if (
            not is_encoder_decoder
            and model_kwargs.get("attention_mask", None) is not None
        ):
            # update attention mask
            attention_mask = model_kwargs["attention_mask"]
            model_kwargs["attention_mask"] = paddle.concat(
                [
                    attention_mask,
                    paddle.ones(
                        [attention_mask.shape[0], 1], dtype=attention_mask.dtype
                    ),
                ],
                axis=-1,
            )

        return model_kwargs

    def get_transpose_weight_keys(self):
        t_layers = [
            "out_proj",
            "q_proj",
            "k_proj",
            "v_proj",
            "lm_head",
            "gate_proj",
            "up_proj",
            "down_proj",
            "o_proj",
            "lm_head",
            "linear_1",
            "linear_2",
            "fc",
            "in_proj",
        ]
        keys = []
        for key, _ in self.get_hf_state_dict().items():
            for t_layer in t_layers:
                if t_layer in key and key.endswith("weight"):
                    keys.append(key)
        return keys

    def get_hf_state_dict(self, *args, **kwargs):
        def _merge_attention_weights(
            q_weight=None,
            k_weight=None,
            v_weight=None,
            q_bias=None,
            k_bias=None,
            v_bias=None,
        ):
            if q_weight is not None and k_weight is not None and v_weight is not None:
                return paddle.concat([q_weight, k_weight, v_weight], axis=1)
            elif q_bias is not None and k_bias is not None and v_bias is not None:
                return paddle.concat([q_bias, k_bias, v_bias], axis=0)
            else:
                raise ValueError

        def _convert_to_hf_state_dict(current_state_dict):
            hf_state_dict = {}

            for key in list(current_state_dict.keys()):
                if "up_gate_proj" in key:
                    combined_weights = current_state_dict[key]
                    split_size = combined_weights.shape[-1] // 2
                    gate_proj = combined_weights[..., :split_size]
                    up_proj = combined_weights[..., split_size:]

                    hf_state_dict[key.replace("up_gate_proj", "gate_proj")] = gate_proj
                    hf_state_dict[key.replace("up_gate_proj", "up_proj")] = up_proj
                    continue

                if "qkv_proj" in key and ("weight" in key or "bias" in key):
                    combined_weights = current_state_dict[key]
                    if getattr(self.config, "head_dim", None) is None:
                        head_dim = self.hidden_size // self.num_heads
                    else:
                        head_dim = self.config.head_dim
                    num_heads = self.config.num_attention_heads
                    num_kv_heads = self.config.num_key_value_heads
                    q_proj, k_proj, v_proj = paddle.split(
                        combined_weights,
                        [
                            num_heads * head_dim,
                            num_kv_heads * head_dim,
                            num_kv_heads * head_dim,
                        ],
                        axis=-1,
                    )

                    if "weight" in key:
                        hf_state_dict[
                            key.replace("qkv_proj.weight", "q_proj.weight")
                        ] = q_proj
                        hf_state_dict[
                            key.replace("qkv_proj.weight", "k_proj.weight")
                        ] = k_proj
                        hf_state_dict[
                            key.replace("qkv_proj.weight", "v_proj.weight")
                        ] = v_proj
                    else:  # bias
                        hf_state_dict[key.replace("qkv_proj.bias", "q_proj.bias")] = (
                            q_proj
                        )
                        hf_state_dict[key.replace("qkv_proj.bias", "k_proj.bias")] = (
                            k_proj
                        )
                        hf_state_dict[key.replace("qkv_proj.bias", "v_proj.bias")] = (
                            v_proj
                        )
                    continue

                if "up_gate_proj" not in key and "qkv_proj" not in key:
                    hf_state_dict[key] = current_state_dict[key]

            new_hf_state_dict = {}
            keys_to_remove = set()

            for key, value in hf_state_dict.items():
                if "head.attention" in key and "out_proj" not in key:
                    if "weight" in key:
                        q_key = key
                        k_key = key.replace("q_proj", "k_proj")
                        v_key = key.replace("q_proj", "v_proj")

                        if (
                            q_key in hf_state_dict
                            and k_key in hf_state_dict
                            and v_key in hf_state_dict
                        ):
                            merged_weights = _merge_attention_weights(
                                q_weight=hf_state_dict[q_key],
                                k_weight=hf_state_dict[k_key],
                                v_weight=hf_state_dict[v_key],
                            )
                            new_key = key.replace("q_proj.weight", "in_proj_weight")
                            new_hf_state_dict[new_key] = merged_weights
                            keys_to_remove.update([q_key, k_key, v_key])

                    elif "bias" in key:
                        q_key = key
                        k_key = key.replace("q_proj", "k_proj")
                        v_key = key.replace("q_proj", "v_proj")

                        if (
                            q_key in hf_state_dict
                            and k_key in hf_state_dict
                            and v_key in hf_state_dict
                        ):
                            merged_bias = _merge_attention_weights(
                                q_bias=hf_state_dict[q_key],
                                k_bias=hf_state_dict[k_key],
                                v_bias=hf_state_dict[v_key],
                            )
                            new_key = key.replace("q_proj.bias", "in_proj_bias")
                            new_hf_state_dict[new_key] = merged_bias
                            keys_to_remove.update([q_key, k_key, v_key])
                else:
                    new_hf_state_dict[key] = value

            for key in keys_to_remove:
                if key in new_hf_state_dict:
                    del new_hf_state_dict[key]

            return new_hf_state_dict

        current_state_dict = self.state_dict(*args, **kwargs)

        hf_state_dict = _convert_to_hf_state_dict(current_state_dict)

        return hf_state_dict

    def set_hf_state_dict(self, state_dict, *args, **kwargs):
        def _split_attention_weights(weight=None, bias=None):
            if weight is not None:
                split_size = weight.shape[1] // 3
                q_weight = weight[:, :split_size]
                k_weight = weight[:, split_size : 2 * split_size]
                v_weight = weight[:, 2 * split_size :]
                return q_weight, k_weight, v_weight
            elif bias is not None:
                split_size = bias.shape[0] // 3
                q_bias = bias[:split_size]
                k_bias = bias[split_size : 2 * split_size]
                v_bias = bias[2 * split_size :]
                return q_bias, k_bias, v_bias

        def _convert_state_dict(old_state_dict):
            new_state_dict = {}
            for key, value in old_state_dict.items():
                if "head.attention.in_proj" in key:
                    if key.endswith("weight"):
                        q_w, k_w, v_w = _split_attention_weights(weight=value)
                        new_state_dict[
                            key.replace("in_proj_weight", "q_proj.weight")
                        ] = q_w
                        new_state_dict[
                            key.replace("in_proj_weight", "k_proj.weight")
                        ] = k_w
                        new_state_dict[
                            key.replace("in_proj_weight", "v_proj.weight")
                        ] = v_w
                    elif key.endswith("bias"):
                        q_b, k_b, v_b = _split_attention_weights(bias=value)
                        new_state_dict[key.replace("in_proj_bias", "q_proj.bias")] = q_b
                        new_state_dict[key.replace("in_proj_bias", "k_proj.bias")] = k_b
                        new_state_dict[key.replace("in_proj_bias", "v_proj.bias")] = v_b
                    else:
                        raise ValueError(f"Unexpected key: {key}")
                else:
                    new_state_dict[key] = value

            for key in list(new_state_dict.keys()):
                if key.startswith("model."):
                    if "mlp.gate_proj." in key:
                        gate_proj = new_state_dict.pop(key)
                        up_proj = new_state_dict.pop(
                            key.replace("gate_proj", "up_proj")
                        )
                        new_state_dict[key.replace("gate_proj", "up_gate_proj")] = (
                            paddle.concat([gate_proj, up_proj], axis=-1)
                        )

                    if "self_attn.q_proj" in key:
                        q_proj = new_state_dict.pop(key)
                        k_proj = new_state_dict.pop(key.replace("q_proj", "k_proj"))
                        v_proj = new_state_dict.pop(key.replace("q_proj", "v_proj"))
                        new_state_dict[key.replace("q_proj", "qkv_proj")] = (
                            paddle.concat([q_proj, k_proj, v_proj], axis=-1)
                        )

            return new_state_dict

        state_dict = _convert_state_dict(state_dict)

        std_state_dict = self.state_dict()
        assert std_state_dict.keys() == state_dict.keys()
        for key in std_state_dict:
            v1 = std_state_dict[key]
            state_dict[key] = state_dict[key].to(v1.place)

        return self.set_state_dict(state_dict, *args, **kwargs)

    def forward(
        self,
        input_ids: paddle.Tensor = None,
        attention_mask: Optional[paddle.Tensor] = None,
        position_ids: Optional[paddle.Tensor] = None,
        past_key_values: Optional[List[paddle.Tensor]] = None,
        inputs_embeds: Optional[paddle.Tensor] = None,
        labels: Optional[paddle.Tensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
        pixel_values: Optional[paddle.Tensor] = None,
        pixel_values_videos: Optional[paddle.Tensor] = None,
        image_grid_thw: Optional[paddle.Tensor] = None,
        video_grid_thw: Optional[paddle.Tensor] = None,
        rope_deltas: Optional[paddle.Tensor] = None,
        second_per_grid_ts: Optional[paddle.Tensor] = None,
        **kwargs,
    ) -> Union[Tuple, PaddleOCRVLCausalLMOutputWithPast]:
        output_attentions = (
            output_attentions
            if output_attentions is not None
            else self.config.output_attentions
        )
        output_hidden_states = (
            output_hidden_states
            if output_hidden_states is not None
            else self.config.output_hidden_states
        )
        return_dict = (
            return_dict if return_dict is not None else self.config.use_return_dict
        )

        curr_rope_deltas = self.rope_deltas_var.get()

        if inputs_embeds is None:
            if input_ids.shape[0] != 1:
                raise NotImplementedError
            inputs_embeds = self.model.embed_tokens(input_ids)
            if pixel_values is not None:
                pixel_values = pixel_values.astype(inputs_embeds.dtype)
                pixel_values = pixel_values.unsqueeze(0)
                siglip_position_ids = list()
                image_grid_hws = list()
                sample_indices = list()
                cu_seqlens = [0]

                pro = 0
                for idx, thw in enumerate(image_grid_thw):
                    thw_tuple = tuple(thw.detach().cpu().numpy().tolist())
                    numel = np.prod(thw_tuple)
                    image_grid_hws.append(thw_tuple)
                    image_position_ids = paddle.arange(numel) % np.prod(thw_tuple[1:])
                    siglip_position_ids.append(image_position_ids)
                    sample_indices.append(
                        paddle.full((numel,), idx, dtype=paddle.int64)
                    )
                    cu_seqlens.append(cu_seqlens[-1] + numel)

                siglip_position_ids = paddle.concat(siglip_position_ids, axis=0)
                cu_seqlens = paddle.to_tensor(cu_seqlens, dtype=paddle.int32)
                sample_indices = paddle.concat(sample_indices, axis=0)

                vision_outputs = self.visual(
                    pixel_values=pixel_values,
                    image_grid_thw=image_grid_hws,
                    position_ids=siglip_position_ids,
                    vision_return_embed_list=True,
                    interpolate_pos_encoding=True,
                    sample_indices=sample_indices,
                    cu_seqlens=cu_seqlens,
                    return_pooler_output=False,
                    use_rope=True,
                    window_size=-1,
                )
                image_embeds = vision_outputs.last_hidden_state

                image_embeds = self.mlp_AR(image_embeds, image_grid_thw)

                n_image_tokens = (input_ids == self.config.image_token_id).sum().item()
                image_embeds = paddle.concat(image_embeds, axis=0)
                n_image_features = image_embeds.shape[0]
                if n_image_tokens != n_image_features:
                    raise ValueError(
                        f"Image features and image tokens do not match: tokens: {n_image_tokens}, features {n_image_features}"
                    )

                mask = input_ids == self.config.image_token_id
                mask_unsqueezed = mask.unsqueeze(-1)
                mask_expanded = mask_unsqueezed.expand_as(inputs_embeds)
                image_mask = mask_expanded

                image_embeds = image_embeds.astype(inputs_embeds.dtype)

                inputs_embeds = inputs_embeds.masked_scatter(image_mask, image_embeds)
        else:
            if inputs_embeds.shape[0] != 1:
                raise NotImplementedError

        if attention_mask is not None and attention_mask.dtype != paddle.bool:
            attention_mask = paddle.cast(attention_mask, paddle.bool)

        # position_ids = None
        # if we get 4D attention mask we cannot calculate rope deltas anymore. TODO @raushan fixme
        if position_ids is None and (
            attention_mask is None or attention_mask.ndim == 2
        ):
            # calculate RoPE index once per generation in the pre-fill stage only
            if curr_rope_deltas is None or (
                past_key_values is None or past_key_values[0] is None
            ):
                position_ids, rope_deltas = self.get_rope_index(
                    input_ids,
                    image_grid_thw,
                    video_grid_thw,
                    second_per_grid_ts,
                    attention_mask,
                )
                self.rope_deltas_var.set(rope_deltas)
            # then use the prev pre-calculated rope-deltas to get the correct position ids
            else:
                batch_size, seq_length, _ = inputs_embeds.shape
                delta = (
                    (past_key_values[0][0].shape[1] + curr_rope_deltas)
                    if past_key_values is not None and past_key_values[0] is not None
                    else 0
                )
                position_ids = paddle.arange(seq_length)
                position_ids = position_ids.reshape((1, -1)).expand((batch_size, -1))
                if (
                    past_key_values is not None and past_key_values[0] is not None
                ):  # otherwise `deltas` is an int `0`
                    delta = delta.repeat_interleave(
                        batch_size // delta.shape[0], axis=0
                    )
                position_ids = position_ids.add(delta)
                position_ids = position_ids.unsqueeze(0).expand((3, -1, -1))

        outputs = self.model(
            input_ids=None,
            position_ids=position_ids,
            attention_mask=attention_mask,
            past_key_values=past_key_values,
            inputs_embeds=inputs_embeds,
            use_cache=use_cache,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
            **kwargs,
        )

        hidden_states = outputs[0]
        logits = self.lm_head(hidden_states)

        loss = None
        if labels is not None:
            # Upcast to float if we need to compute the loss to avoid potential precision issues
            logits = logits.astype("float32")
            # Shift so that tokens < n predict n
            shift_logits = logits[..., :-1, :].contiguous()
            shift_labels = labels[..., 1:].contiguous()
            # Flatten the tokens
            loss_fct = paddle.nn.CrossEntropyLoss()
            shift_logits = shift_logits.reshape((-1, self.config.vocab_size))
            shift_labels = shift_labels.reshape((-1,))
            loss = loss_fct(shift_logits, shift_labels)

        if not return_dict:
            output = (logits,) + outputs[1:]
            return (loss,) + output if loss is not None else output

        return PaddleOCRVLCausalLMOutputWithPast(
            loss=loss,
            logits=logits,
            past_key_values=outputs.past_key_values,
            hidden_states=outputs.hidden_states,
            attentions=outputs.attentions,
            rope_deltas=curr_rope_deltas,
        )

    def generate(self, inputs, **kwargs):
        gen_kwargs = {
            "max_new_tokens": kwargs.get("max_new_tokens", 8192),
            "use_cache": kwargs.get("use_cache", True),
        }
        gen_kwargs = {**inputs, **gen_kwargs}
        with paddle.no_grad():
            generated_ids = super().generate(**gen_kwargs)
        return generated_ids

    def _get_image_nums_and_video_nums(
        self,
        input_ids: Optional[paddle.Tensor],
    ) -> Tuple[paddle.Tensor, paddle.Tensor]:
        """
        Get the number of images and videos for each sample to calculate the separation length of the sample tensor.
        These parameters are not passed through the processor to avoid unpredictable impacts from interface modifications.

        Args:
            input_ids (`paddle.Tensor` of shape `(batch_size, sequence_length)`):
                Indices of input sequence tokens in the vocabulary.

        Returns:
            image_nums (`paddle.Tensor` of shape `(batch_size, num_images_sample)`)
            video_nums (`paddle.Tensor` of shape `(batch_size, num_videos_sample)`)
        """
        image_token_id = self.config.image_token_id
        video_token_id = self.config.video_token_id
        vision_start_token_id = self.config.vision_start_token_id

        vision_start_mask = input_ids == vision_start_token_id
        vision_first_mask = paddle.roll(vision_start_mask, shifts=1, axis=1)
        image_mask = input_ids == image_token_id
        video_mask = input_ids == video_token_id
        image_nums = paddle.sum(vision_first_mask & image_mask, axis=1)
        video_nums = paddle.sum(vision_first_mask & video_mask, axis=1)

        return image_nums, video_nums



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from .... import UltraInferModel, ModelFormat
from .... import c_lib_wrap as C


class MODNet(UltraInferModel):
    def __init__(
        self,
        model_file,
        params_file="",
        runtime_option=None,
        model_format=ModelFormat.ONNX,
    ):
        """Load a MODNet model exported by MODNet.

        :param model_file: (str)Path of model file, e.g ./modnet.onnx
        :param params_file: (str)Path of parameters file, e.g yolox/model.pdiparams, if the model_fomat is ModelFormat.ONNX, this param will be ignored, can be set as empty string
        :param runtime_option: (ultra_infer.RuntimeOption)RuntimeOption for inference this model, if it's None, will use the default backend on CPU
        :param model_format: (ultra_infer.ModelForamt)Model format of the loaded model
        """
        # 调用基函数进行backend_option的初始化
        # 初始化后的option保存在self._runtime_option
        super(MODNet, self).__init__(runtime_option)

        self._model = C.vision.matting.MODNet(
            model_file, params_file, self._runtime_option, model_format
        )
        # 通过self.initialized判断整个模型的初始化是否成功
        assert self.initialized, "MODNet initialize failed."

    def predict(self, input_image):
        """Predict the matting result for an input image

        :param input_image: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
        :return: MattingResult
        """
        return self._model.predict(input_image)

    # 一些跟模型有关的属性封装
    # 多数是预处理相关，可通过修改如model.size = [256, 256]改变预处理时resize的大小（前提是模型支持）
    @property
    def size(self):
        """
        Argument for image preprocessing step, the preprocess image size, tuple of (width, height), default size = [256,256]
        """
        return self._model.size

    @property
    def alpha(self):
        """
        Argument for image preprocessing step, alpha value for normalization, default alpha = {1.f / 127.5f, 1.f / 127.5f, 1.f / 127.5f}
        """
        return self._model.alpha

    @property
    def beta(self):
        """
        Argument for image preprocessing step, beta value for normalization, default beta = {-1.f, -1.f, -1.f}
        """
        return self._model.beta

    @property
    def swap_rb(self):
        """
        Argument for image preprocessing step, whether to swap the B and R channel, such as BGR->RGB, default True.
        """
        return self._model.swap_rb

    @size.setter
    def size(self, wh):
        assert isinstance(
            wh, (list, tuple)
        ), "The value to set `size` must be type of tuple or list."
        assert (
            len(wh) == 2
        ), "The value to set `size` must contains 2 elements means [width, height], but now it contains {} elements.".format(
            len(wh)
        )
        self._model.size = wh

    @alpha.setter
    def alpha(self, value):
        assert isinstance(
            value, (list, tuple)
        ), "The value to set `alpha` must be type of tuple or list."
        assert (
            len(value) == 3
        ), "The value to set `alpha` must contains 3 elements for each channels, but now it contains {} elements.".format(
            len(value)
        )
        self._model.alpha = value

    @beta.setter
    def beta(self, value):
        assert isinstance(
            value, (list, tuple)
        ), "The value to set `beta` must be type of tuple or list."
        assert (
            len(value) == 3
        ), "The value to set `beta` must contains 3 elements for each channels, but now it contains {} elements.".format(
            len(value)
        )
        self._model.beta = value

    @swap_rb.setter
    def swap_rb(self, value):
        assert isinstance(
            value, bool
        ), "The value to set `swap_rb` must be type of bool."
        self._model.swap_rb = value



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import numpy as np
import os
import time
import collections


def topk_accuracy(topk_list, label_list):
    match_array = np.logical_or.reduce(topk_list == label_list, axis=1)
    topk_acc_score = match_array.sum() / match_array.shape[0]
    return topk_acc_score


def eval_classify(model, image_file_path, label_file_path, topk=5):
    from tqdm import trange
    import cv2
    import math

    result_list = []
    label_list = []
    image_label_dict = {}
    assert os.path.isdir(
        image_file_path
    ), "The image_file_path:{} is not a directory.".format(image_file_path)
    assert os.path.isfile(
        label_file_path
    ), "The label_file_path:{} is not a file.".format(label_file_path)
    assert isinstance(topk, int), "The tok:{} is not int type".format(topk)

    with open(label_file_path, "r") as file:
        lines = file.readlines()
        for line in lines:
            items = line.strip().split()
            image_name = items[0]
            label = items[1]
            image_label_dict[image_name] = int(label)
    images_num = len(image_label_dict)
    twenty_percent_images_num = math.ceil(images_num * 0.2)
    start_time = 0
    end_time = 0
    average_inference_time = 0
    scores = collections.OrderedDict()
    for (image, label), i in zip(
        image_label_dict.items(), trange(images_num, desc="Inference Progress")
    ):
        if i == twenty_percent_images_num:
            start_time = time.time()

        label_list.append([label])
        image_path = os.path.join(image_file_path, image)
        im = cv2.imread(image_path)
        result = model.predict(im, topk)
        result_list.append(result.label_ids)
        if i == images_num - 1:
            end_time = time.time()
    average_inference_time = round(
        (end_time - start_time) / (images_num - twenty_percent_images_num), 4
    )
    topk_acc_score = topk_accuracy(np.array(result_list), np.array(label_list))
    if topk == 1:
        scores.update({"topk1": topk_acc_score})
        scores.update({"topk1_average_inference_time(s)": average_inference_time})
    elif topk == 5:
        scores.update({"topk5": topk_acc_score})
        scores.update({"topk5_average_inference_time(s)": average_inference_time})
    return scores



#!/usr/bin/env python

# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import json
import pathlib
import shutil
import subprocess
import sys
import tarfile
import tempfile

TARGET_NAME_PATTERN = "paddlex_hps_{pipeline_name}_sdk"
ARCHIVE_SUFFIX = ".tar.gz"

BASE_DIR = pathlib.Path.cwd()
PIPELINES_DIR = BASE_DIR / "pipelines"
COMMON_DIR = BASE_DIR / "common"
CLIENT_LIB_PATH = BASE_DIR / "paddlex-hps-client"
VERSIONS_PATH = BASE_DIR / "versions.json"
OUTPUT_DIR = BASE_DIR / "output"


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("pipeline_names", type=str, metavar="pipeline-names", nargs="*")
    parser.add_argument("--all", action="store_true")
    parser.add_argument(
        "--no-server",
        action="store_true",
    )
    parser.add_argument(
        "--no-client",
        action="store_true",
    )
    args = parser.parse_args()

    if args.all and args.pipeline_names:
        print(
            "Cannot specify `--all` and `pipeline-names` at the same time",
            file=sys.stderr,
        )
        sys.exit(2)

    if args.all:
        pipeline_names = [p.name for p in PIPELINES_DIR.iterdir()]
    else:
        pipeline_names = args.pipeline_names

    if not pipeline_names:
        sys.exit(0)

    with_server = not args.no_server
    with_client = not args.no_client

    OUTPUT_DIR.mkdir(exist_ok=True)

    if with_client:
        # HACK: Make a copy to avoid creating files in the source directory
        with tempfile.TemporaryDirectory() as td:
            tmp_client_lib_path = shutil.copytree(
                CLIENT_LIB_PATH, str(pathlib.Path(td, CLIENT_LIB_PATH.name))
            )
            subprocess.check_call(
                [
                    sys.executable,
                    "-m",
                    "pip",
                    "wheel",
                    "--no-deps",
                    "--wheel-dir",
                    str(OUTPUT_DIR),
                    tmp_client_lib_path,
                ]
            )
            client_lib_whl_path = next(OUTPUT_DIR.glob("paddlex_hps_client*.whl"))

    with VERSIONS_PATH.open("r", encoding="utf-8") as f:
        versions = json.load(f)

    for pipeline_name in pipeline_names:
        print("=" * 30)
        print(f"Pipeline: {pipeline_name}")
        pipeline_dir = PIPELINES_DIR / pipeline_name
        if not pipeline_dir.exists():
            sys.exit(f"{pipeline_dir} not found")
        if pipeline_name not in versions:
            sys.exit(f"Version is missing for {repr(pipeline_name)}")

        tgt_name = TARGET_NAME_PATTERN.format(pipeline_name=pipeline_name)
        tgt_dir = OUTPUT_DIR / tgt_name

        if tgt_dir.exists():
            print(f"Removing existing target directory: {tgt_dir}")
            shutil.rmtree(tgt_dir)

        if with_server:
            shutil.copytree(pipeline_dir / "server", tgt_dir / "server")
            shutil.copy(COMMON_DIR / "server.sh", tgt_dir / "server")
            for dir_ in (tgt_dir / "server" / "model_repo").iterdir():
                if dir_.is_dir():
                    if (dir_ / "config.pbtxt").exists():
                        continue
                    for device_type in ("cpu", "gpu"):
                        config_path = dir_ / f"config_{device_type}.pbtxt"
                        if not config_path.exists():
                            shutil.copy(
                                COMMON_DIR / f"config_{device_type}.pbtxt", config_path
                            )

        if with_client:
            shutil.copytree(pipeline_dir / "client", tgt_dir / "client")
            shutil.copy(client_lib_whl_path, tgt_dir / "client")

        version = versions[pipeline_name]
        (tgt_dir / "version.txt").write_text(version + "\n", encoding="utf-8")

        arch_path = tgt_dir.with_suffix(ARCHIVE_SUFFIX)
        print(f"Creating archive: {arch_path}")
        with tarfile.open(arch_path, "w:gz") as tar:
            tar.add(tgt_dir, arcname=tgt_dir.name)
        print("Done" + "\n" + "=" * 30)



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import numpy as np


def get_det_res(bboxes, bbox_nums, image_id, label_to_cat_id_map, bias=0):
    det_res = []
    for i in range(bbox_nums):
        cur_image_id = int(image_id)
        dt = bboxes[i]
        num_id, score, xmin, ymin, xmax, ymax = dt
        if int(num_id) < 0:
            continue
        category_id = label_to_cat_id_map[int(num_id)]
        w = xmax - xmin + bias
        h = ymax - ymin + bias
        bbox = [xmin, ymin, w, h]
        dt_res = {
            "image_id": cur_image_id,
            "category_id": category_id,
            "bbox": bbox,
            "score": score,
        }
        det_res.append(dt_res)
    return det_res


def get_det_poly_res(bboxes, bbox_nums, image_id, label_to_cat_id_map, bias=0):
    det_res = []
    k = 0
    for i in range(len(bbox_nums)):
        cur_image_id = int(image_id[i][0])
        det_nums = bbox_nums[i]
        for j in range(det_nums):
            dt = bboxes[k]
            k = k + 1
            num_id, score, x1, y1, x2, y2, x3, y3, x4, y4 = dt.tolist()
            if int(num_id) < 0:
                continue
            category_id = label_to_cat_id_map[int(num_id)]
            rbox = [x1, y1, x2, y2, x3, y3, x4, y4]
            dt_res = {
                "image_id": cur_image_id,
                "category_id": category_id,
                "bbox": rbox,
                "score": score,
            }
            det_res.append(dt_res)
    return det_res


def strip_mask(mask):
    row = mask[0, 0, :]
    col = mask[0, :, 0]
    im_h = len(col) - np.count_nonzero(col == -1)
    im_w = len(row) - np.count_nonzero(row == -1)
    return mask[:, :im_h, :im_w]


def get_seg_res(masks, bboxes, mask_nums, image_id, label_to_cat_id_map):
    import pycocotools.mask as mask_util

    seg_res = []
    k = 0
    for i in range(len(mask_nums)):
        cur_image_id = int(image_id[i][0])
        det_nums = mask_nums[i]
        mask_i = masks[k : k + det_nums]
        mask_i = strip_mask(mask_i)
        for j in range(det_nums):
            mask = mask_i[j].astype(np.uint8)
            score = float(bboxes[k][1])
            label = int(bboxes[k][0])
            k = k + 1
            if label == -1:
                continue
            cat_id = label_to_cat_id_map[label]
            rle = mask_util.encode(
                np.array(mask[:, :, None], order="F", dtype="uint8")
            )[0]
            if six.PY3:
                if "counts" in rle:
                    rle["counts"] = rle["counts"].decode("utf8")
            sg_res = {
                "image_id": cur_image_id,
                "category_id": cat_id,
                "segmentation": rle,
                "score": score,
            }
            seg_res.append(sg_res)
    return seg_res


def get_solov2_segm_res(results, image_id, num_id_to_cat_id_map):
    import pycocotools.mask as mask_util

    segm_res = []
    # for each batch
    segms = results["segm"].astype(np.uint8)
    clsid_labels = results["cate_label"]
    clsid_scores = results["cate_score"]
    lengths = segms.shape[0]
    im_id = int(image_id[0][0])
    if lengths == 0 or segms is None:
        return None
    # for each sample
    for i in range(lengths - 1):
        clsid = int(clsid_labels[i])
        catid = num_id_to_cat_id_map[clsid]
        score = float(clsid_scores[i])
        mask = segms[i]
        segm = mask_util.encode(np.array(mask[:, :, np.newaxis], order="F"))[0]
        segm["counts"] = segm["counts"].decode("utf8")
        coco_res = {
            "image_id": im_id,
            "category_id": catid,
            "segmentation": segm,
            "score": score,
        }
        segm_res.append(coco_res)
    return segm_res


def get_keypoint_res(results, im_id):
    anns = []
    preds = results["keypoint"]
    for idx in range(im_id.shape[0]):
        image_id = im_id[idx].item()
        kpts, scores = preds[idx]
        for kpt, score in zip(kpts, scores):
            kpt = kpt.flatten()
            ann = {
                "image_id": image_id,
                "category_id": 1,  # XXX hard code
                "keypoints": kpt.tolist(),
                "score": float(score),
            }
            x = kpt[0::3]
            y = kpt[1::3]
            x0, x1, y0, y1 = (
                np.min(x).item(),
                np.max(x).item(),
                np.min(y).item(),
                np.max(y).item(),
            )
            ann["area"] = (x1 - x0) * (y1 - y0)
            ann["bbox"] = [x0, y0, x1 - x0, y1 - y0]
            anns.append(ann)
    return anns



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
import logging
from ... import c_lib_wrap as C
import cv2


def vis_detection(
    im_data,
    det_result,
    labels=[],
    score_threshold=0.0,
    line_size=1,
    font_size=0.5,
    font_color=[255, 255, 255],
    font_thickness=1,
):
    """Show the visualized results for detection models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param det_result: the result produced by model
    :param labels: (list of str) the visualized result will show the bounding box contain class label
    :param score_threshold: (float) score_threshold threshold for result scores, the bounding box will not be shown if the score is less than score_threshold
    :param line_size: (float) line_size line size for bounding boxes
    :param font_size: (float) font_size font size for text
    :param font_color: (list of int) font_color  for text
    :param font_thickness: (int) font_thickness for text
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_detection(
        im_data,
        det_result,
        labels,
        score_threshold,
        line_size,
        font_size,
        font_color,
        font_thickness,
    )


def vis_perception(
    im_data, det_result, config_file, score_threshold=0.0, line_size=1, font_size=0.5
):
    """Show the visualized results for 3d detection models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param det_result: the result produced by model
    :param config_file: the config file for detection and visualization
    :param score_threshold: (float) score_threshold threshold for result scores, the bounding box will not be shown if the score is less than score_threshold
    :param line_size: (float) line_size line size for bounding boxes
    :param font_size: (float) font_size font size for text
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_perception(
        im_data, det_result, config_file, score_threshold, line_size, font_size
    )


def vis_keypoint_detection(im_data, keypoint_det_result, conf_threshold=0.5):
    """Show the visualized results for keypoint detection models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param keypoint_det_result: the result produced by model
    :param conf_threshold: (float) conf_threshold threshold for result scores, the bounding box will not be shown if the score is less than conf_threshold
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.Visualize.vis_keypoint_detection(
        im_data, keypoint_det_result, conf_threshold
    )


def vis_face_detection(im_data, face_det_result, line_size=1, font_size=0.5):
    """Show the visualized results for face detection models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param face_det_result: the result produced by model
    :param line_size: (float) line_size line size for bounding boxes
    :param font_size: (float) font_size font size for text
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_face_detection(im_data, face_det_result, line_size, font_size)


def vis_face_alignment(im_data, face_align_result, line_size=1):
    """Show the visualized results for face alignment models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param face_align_result: the result produced by model
    :param line_size: (float)line_size line size for circle point
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_face_alignment(im_data, face_align_result, line_size)


def vis_segmentation(im_data, seg_result, weight=0.5):
    """Show the visualized results for segmentation models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param seg_result: the result produced by model
    :param weight: (float)transparent weight of visualized result image
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_segmentation(im_data, seg_result, weight)


def vis_matting_alpha(im_data, matting_result, remove_small_connected_area=False):
    logging.warning(
        "DEPRECATED: ultra_infer.vision.vis_matting_alpha is deprecated, please use ultra_infer.vision.vis_matting function instead."
    )
    return C.vision.vis_matting(im_data, matting_result, remove_small_connected_area)


def vis_matting(
    im_data,
    matting_result,
    transparent_background=False,
    transparent_threshold=0.99,
    remove_small_connected_area=False,
):
    """Show the visualized results for matting models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param matting_result: the result produced by model
    :param transparent_background: whether visualizing matting result with transparent background
    :param transparent_threshold: since the alpha value in MattringResult is a float between [0, 1], transparent_threshold is used to filter background pixel
    :param remove_small_connected_area: (bool) if remove_small_connected_area==True, the visualized result will not include the small connected areas
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_matting(
        im_data,
        matting_result,
        transparent_background,
        transparent_threshold,
        remove_small_connected_area,
    )


def swap_background_matting(
    im_data, background, result, remove_small_connected_area=False
):
    logging.warning(
        "DEPRECATED: ultra_infer.vision.swap_background_matting is deprecated, please use ultra_infer.vision.swap_background function instead."
    )
    assert isinstance(
        result, C.vision.MattingResult
    ), "The result must be MattingResult type"
    return C.vision.Visualize.swap_background_matting(
        im_data, background, result, remove_small_connected_area
    )


def swap_background_segmentation(im_data, background, background_label, result):
    logging.warning(
        "DEPRECATED: ultra_infer.vision.swap_background_segmentation is deprecated, please use ultra_infer.vision.swap_background function instead."
    )
    assert isinstance(
        result, C.vision.SegmentationResult
    ), "The result must be SegmentaitonResult type"
    return C.vision.Visualize.swap_background_segmentation(
        im_data, background, background_label, result
    )


def swap_background(
    im_data, background, result, remove_small_connected_area=False, background_label=0
):
    """Swap the image background with MattingResult or SegmentationResult

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param background: (numpy.ndarray)The background image data, 3-D array with layout HWC, BGR format
    :param result: The result produced by model, MattingResult or SegmentationResult
    :param remove_small_connected_area: (bool) If remove_small_connected_area==True, the visualized result will not include the small connected areas
    :param background_label: (int)The background label number in SegmentationResult
    :return: (numpy.ndarray) image with visualized results
    """
    if isinstance(result, C.vision.MattingResult):
        return C.vision.swap_background(
            im_data, background, result, remove_small_connected_area
        )
    elif isinstance(result, C.vision.SegmentationResult):
        return C.vision.swap_background(im_data, background, result, background_label)
    else:
        raise Exception(
            "Only support result type of MattingResult or SegmentationResult, but now the data type is {}.".format(
                type(result)
            )
        )


def vis_ppocr(im_data, det_result):
    """Show the visualized results for ocr models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param det_result: the result produced by model
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_ppocr(im_data, det_result)


def vis_ppocr_curve(im_data, det_result):
    """Show the visualized results for ocr models

    :param im_data: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
    :param det_result: the result produced by model
    :return: (numpy.ndarray) image with visualized results
    """
    return C.vision.vis_ppocr_curve(im_data, det_result)


def vis_mot(im_data, mot_result, score_threshold=0.0, records=None):
    return C.vision.vis_mot(im_data, mot_result, score_threshold, records)


def vis_headpose(im_data, headpose_result, size=50, line_size=1):
    return C.vision.vis_headpose(im_data, headpose_result, size, line_size)



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from paddlex_hps_server import BaseTritonPythonModel, schemas


class TritonPythonModel(BaseTritonPythonModel):
    @property
    def pipeline_creation_kwargs(self):
        return {"initial_predictor": False}

    def get_input_model_type(self):
        return schemas.pp_chatocrv3_doc.BuildVectorStoreRequest

    def get_result_model_type(self):
        return schemas.pp_chatocrv3_doc.BuildVectorStoreResult

    def run(self, input, log_id):
        vector_info = self.pipeline.build_vector(
            input.visualInfo,
            min_characters=input.minCharacters,
            block_size=input.blockSize,
            flag_save_bytes_vector=True,
            retriever_config=input.retrieverConfig,
        )

        return schemas.pp_chatocrv3_doc.BuildVectorStoreResult(vectorInfo=vector_info)



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import glob
from . import fd_logging as logging

# import fd_logging as logging


class Cityscapes(object):
    """
    Cityscapes dataset `https://www.cityscapes-dataset.com/`.
    The folder structure is as follow:

        cityscapes
        |
        |--leftImg8bit
        |  |--train
        |  |--val
        |  |--test
        |
        |--gtFine
        |  |--train
        |  |--val
        |  |--test

    Args:
        dataset_root (str): Cityscapes dataset directory.
    """

    NUM_CLASSES = 19

    def __init__(self, dataset_root, mode):
        self.dataset_root = dataset_root
        self.file_list = list()
        mode = mode.lower()
        self.mode = mode
        self.num_classes = self.NUM_CLASSES
        self.ignore_index = 255

        img_dir = os.path.join(self.dataset_root, "leftImg8bit")
        label_dir = os.path.join(self.dataset_root, "gtFine")
        if (
            self.dataset_root is None
            or not os.path.isdir(self.dataset_root)
            or not os.path.isdir(img_dir)
            or not os.path.isdir(label_dir)
        ):
            raise ValueError(
                "The dataset is not Found or the folder structure is nonconfoumance."
            )

        label_files = sorted(
            glob.glob(os.path.join(label_dir, mode, "*", "*_gtFine_labelTrainIds.png"))
        )
        img_files = sorted(
            glob.glob(os.path.join(img_dir, mode, "*", "*_leftImg8bit.png"))
        )

        self.file_list = [
            [img_path, label_path]
            for img_path, label_path in zip(img_files, label_files)
        ]

        self.num_samples = len(self.file_list)
        logging.info("{} samples in file {}".format(self.num_samples, img_dir))



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import os
from copy import deepcopy
import numpy as np
from dataclasses import dataclass

from .... import ModelFormat
from ....py_only.ts import PyOnlyTSModel
from ....utils.misc import load_config
from ....py_only import PyOnlyProcessorChain
from ....py_only.ts import PyOnlyTSModel, processors as P


class PyOnlyClassificationModel(PyOnlyTSModel):
    def __init__(
        self,
        model_file,
        params_file,
        config_file,
        scaler_file=None,
        runtime_option=None,
        model_format=ModelFormat.PADDLE,
    ):
        self._model_file = model_file
        self._params_file = params_file
        self._model_format = model_format
        super().__init__(runtime_option)
        if scaler_file is None:
            config_dir = os.path.dirname(config_file)
            scaler_file = os.path.join(config_dir, "scaler.pkl")
        self._config = load_config(config_file)
        self._preprocessor = _PyOnlyClassificationPreprocessor(
            self._config, scaler_file
        )
        self._postprocessor = _PyOnlyClassificationPostprocessor()

    def model_name():
        return "PyOnlyClassificationModel"

    def batch_predict(self, ts_list):
        data_list = []
        for csv_data in ts_list:
            data = {"ori_ts": deepcopy(csv_data), "ts": csv_data}
            data = self._preprocessor.run(data)
            data_list.append(data)

        input_data = {}
        input_num = self._runtime.num_inputs()
        for idx in range(input_num):
            input_name = self._runtime.get_input_info(idx).name
            ts_data = np.stack(
                [data["ts"][idx] for data in data_list], axis=0, dtype=np.float32
            )
            ts_data = np.ascontiguousarray(ts_data)
            input_data[input_name] = ts_data

        output_arrs = self._runtime.infer(input_data)

        results = []
        for data in output_arrs[0]:
            data = {"pred": data}
            result = self._postprocessor.run(data)
            results.append(result)
        return results

    def _update_option(self):
        self._option.set_model_path(
            self._model_file, self._params_file, self._model_format
        )


class _PyOnlyClassificationPreprocessor(object):
    def __init__(self, config, scaler_file):
        super().__init__()
        self.scaler_file = scaler_file
        processors = self._build_processors(config)
        self._processor_chain = PyOnlyProcessorChain(processors)

    def run(self, data):
        return self._processor_chain(data)

    def _build_processors(self, config):
        processors = []

        if config.get("scale", None):
            if not os.path.exists(self.scaler_file):
                raise Exception(f"Cannot find scaler file: {self.scaler_file}")
            processors.append(P.Normalize(self.scaler_file, config["info_params"]))

        processors.append(P.BuildTSDataset(config["info_params"]))
        processors.append(P.BuildPaddedMask(config["input_data"]))
        processors.append(P.DataFrame2Arrays(config["input_data"]))
        return processors


class _PyOnlyClassificationPostprocessor(object):
    def __init__(self):
        super().__init__()

    def run(self, data):
        pred_ts = data["pred"]
        pred_ts -= np.max(pred_ts, axis=-1, keepdims=True)
        pred_ts = np.exp(pred_ts) / np.sum(np.exp(pred_ts), axis=-1, keepdims=True)
        class_id = np.argmax(pred_ts, axis=-1)
        pred_score = pred_ts[class_id]
        result = _PyOnlyClassificationResult(class_id=class_id, score=pred_score)
        return result


@dataclass
class _PyOnlyClassificationResult(object):
    class_id: int
    score: float



# Copyright (c) 2024 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from .... import UltraInferModel, ModelFormat
from .... import c_lib_wrap as C


class YOLOv5Preprocessor:
    def __init__(self):
        """Create a preprocessor for YOLOv5"""
        self._preprocessor = C.vision.detection.YOLOv5Preprocessor()

    def run(self, input_ims):
        """Preprocess input images for YOLOv5

        :param: input_ims: (list of numpy.ndarray)The input image
        :return: list of FDTensor
        """
        return self._preprocessor.run(input_ims)

    @property
    def size(self):
        """
        Argument for image preprocessing step, the preprocess image size, tuple of (width, height), default size = [640, 640]
        """
        return self._preprocessor.size

    @property
    def padding_value(self):
        """
        padding value for preprocessing, default [114.0, 114.0, 114.0]
        """
        #  padding value, size should be the same as channels
        return self._preprocessor.padding_value

    @property
    def is_scale_up(self):
        """
        is_scale_up for preprocessing, the input image only can be zoom out, the maximum resize scale cannot exceed 1.0, default true
        """
        return self._preprocessor.is_scale_up

    @property
    def is_mini_pad(self):
        """
        is_mini_pad for preprocessing, pad to the minimum rectangle which height and width is times of stride, default false
        """
        return self._preprocessor.is_mini_pad

    @property
    def stride(self):
        """
        stride for preprocessing, only for mini_pad mode, default 32
        """
        return self._preprocessor.stride

    @size.setter
    def size(self, wh):
        assert isinstance(
            wh, (list, tuple)
        ), "The value to set `size` must be type of tuple or list."
        assert (
            len(wh) == 2
        ), "The value to set `size` must contains 2 elements means [width, height], but now it contains {} elements.".format(
            len(wh)
        )
        self._preprocessor.size = wh

    @padding_value.setter
    def padding_value(self, value):
        assert isinstance(
            value, list
        ), "The value to set `padding_value` must be type of list."
        self._preprocessor.padding_value = value

    @is_scale_up.setter
    def is_scale_up(self, value):
        assert isinstance(
            value, bool
        ), "The value to set `is_scale_up` must be type of bool."
        self._preprocessor.is_scale_up = value

    @is_mini_pad.setter
    def is_mini_pad(self, value):
        assert isinstance(
            value, bool
        ), "The value to set `is_mini_pad` must be type of bool."
        self._preprocessor.is_mini_pad = value

    @stride.setter
    def stride(self, value):
        assert isinstance(value, int), "The value to set `stride` must be type of int."
        self._preprocessor.stride = value


class YOLOv5Postprocessor:
    def __init__(self):
        """Create a postprocessor for YOLOv5"""
        self._postprocessor = C.vision.detection.YOLOv5Postprocessor()

    def run(self, runtime_results, ims_info):
        """Postprocess the runtime results for YOLOv5

        :param: runtime_results: (list of FDTensor)The output FDTensor results from runtime
        :param: ims_info: (list of dict)Record input_shape and output_shape
        :return: list of DetectionResult(If the runtime_results is predict by batched samples, the length of this list equals to the batch size)
        """
        return self._postprocessor.run(runtime_results, ims_info)

    @property
    def conf_threshold(self):
        """
        confidence threshold for postprocessing, default is 0.25
        """
        return self._postprocessor.conf_threshold

    @property
    def nms_threshold(self):
        """
        nms threshold for postprocessing, default is 0.5
        """
        return self._postprocessor.nms_threshold

    @property
    def multi_label(self):
        """
        multi_label for postprocessing, set true for eval, default is True
        """
        return self._postprocessor.multi_label

    @conf_threshold.setter
    def conf_threshold(self, conf_threshold):
        assert isinstance(
            conf_threshold, float
        ), "The value to set `conf_threshold` must be type of float."
        self._postprocessor.conf_threshold = conf_threshold

    @nms_threshold.setter
    def nms_threshold(self, nms_threshold):
        assert isinstance(
            nms_threshold, float
        ), "The value to set `nms_threshold` must be type of float."
        self._postprocessor.nms_threshold = nms_threshold

    @multi_label.setter
    def multi_label(self, value):
        assert isinstance(
            value, bool
        ), "The value to set `multi_label` must be type of bool."
        self._postprocessor.multi_label = value


class YOLOv5(UltraInferModel):
    def __init__(
        self,
        model_file,
        params_file="",
        runtime_option=None,
        model_format=ModelFormat.ONNX,
    ):
        """Load a YOLOv5 model exported by YOLOv5.

        :param model_file: (str)Path of model file, e.g ./yolov5.onnx
        :param params_file: (str)Path of parameters file, e.g yolox/model.pdiparams, if the model_fomat is ModelFormat.ONNX, this param will be ignored, can be set as empty string
        :param runtime_option: (ultra_infer.RuntimeOption)RuntimeOption for inference this model, if it's None, will use the default backend on CPU
        :param model_format: (ultra_infer.ModelForamt)Model format of the loaded model
        """
        # 调用基函数进行backend_option的初始化
        # 初始化后的option保存在self._runtime_option
        super(YOLOv5, self).__init__(runtime_option)

        self._model = C.vision.detection.YOLOv5(
            model_file, params_file, self._runtime_option, model_format
        )
        # 通过self.initialized判断整个模型的初始化是否成功
        assert self.initialized, "YOLOv5 initialize failed."

    def predict(self, input_image, conf_threshold=0.25, nms_iou_threshold=0.5):
        """Detect an input image

        :param input_image: (numpy.ndarray)The input image data, 3-D array with layout HWC, BGR format
        :param conf_threshold: confidence threshold for postprocessing, default is 0.25
        :param nms_iou_threshold: iou threshold for NMS, default is 0.5
        :return: DetectionResult
        """

        self.postprocessor.conf_threshold = conf_threshold
        self.postprocessor.nms_threshold = nms_iou_threshold
        return self._model.predict(input_image)

    def batch_predict(self, images):
        """Classify a batch of input image

        :param im: (list of numpy.ndarray) The input image list, each element is a 3-D array with layout HWC, BGR format
        :return list of DetectionResult
        """

        return self._model.batch_predict(images)

    @property
    def preprocessor(self):
        """Get YOLOv5Preprocessor object of the loaded model

        :return YOLOv5Preprocessor
        """
        return self._model.preprocessor

    @property
    def postprocessor(self):
        """Get YOLOv5Postprocessor object of the loaded model

        :return YOLOv5Postprocessor
        """
        return self._model.postprocessor



# Copyright (c) 2025 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from paddlex.inference.serving.infra.utils import (
    base64_encode,
    csv_bytes_to_data_frame,
    data_frame_to_bytes,
    file_to_images,
    get_image_info,
    get_raw_bytes,
    image_array_to_bytes,
    image_bytes_to_array,
    image_bytes_to_image,
    image_to_bytes,
    infer_file_ext,
    infer_file_type,
    is_url,
    read_pdf,
    write_to_temp_file,
)

__all__ = [
    "base64_encode",
    "csv_bytes_to_data_frame",
    "data_frame_to_bytes",
    "file_to_images",
    "get_image_info",
    "get_raw_bytes",
    "image_array_to_bytes",
    "image_bytes_to_array",
    "image_bytes_to_image",
    "image_to_bytes",
    "infer_file_ext",
    "infer_file_type",
    "is_url",
    "read_pdf",
    "write_to_temp_file",
]
